# -*- encoding: iso-8859-1 -*-
#
# Copyright(C)  2004-2007 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL(>=v2)
# Read the file COPYING coming with the software for details.

"""Hier sind die Klassen fr edbsilon.
   Reihenfolge der Hauptsachen:
               globale Variablen
               Klasse edbs_line
               Klasse Linienzug
               Klasse Polygon
               Klasse Loch
               Klasse Feature
               Klasse edbs_satz
               Klasse edbs_satz_liste  """

#fr die Winkelberechnungen
from math import sqrt

import logging
import string
log = logging.getLogger('')

Debug_Ordner = None

#fr die Umwandelung des EDBS-Zeichensatzes
translation = (
    """??????????\n??\n??"""
    """????????????????"""
    """ !"#$%&'()*+,-./"""
    """0123456789:;<=>?"""
    """@ABCDEFGHIJKLMNO"""
    """PQRSTUVWXYZ^_"""
    """`abcdefghijklmno"""
    """pqrstuvwxyz?"""
    """????????????????"""
    """????????????????"""
    """????????????????"""
    """????????????????"""
    """????????????????"""
    """????????????????"""
    """????????????????"""
    """????????????????"""
)


def punkttrans(pktstring, edbsformat):
    """Transferiert einen Punktstring zu einen Paar von x,y-Koordinaten"""

    x = 0.0
    y = 0.0
    x = 100000.0 * int(pktstring[0:2])
    y = 100000.0 * int(pktstring[2:4])
    if edbsformat == 'atkis':
        x +=  1000.0 * int(pktstring[4:5])
        y +=  1000.0 * int(pktstring[6:7])
        x +=     0.01 * int(pktstring[8:14])
        y +=     0.01 * int(pktstring[14:20])
        return(round(x,2),round(y,2))
    else:
        x +=  1000.0 * int(pktstring[4:6])
        y +=  1000.0 * int(pktstring[6:8])
        x +=     0.001 * int(pktstring[8:14])
        y +=     0.001 * int(pktstring[14:20])
        return(round(x,3),round(y,3))
 

class edbs_line:
    """Beinhaltet eine Teillinie eines Objekt oder 
      Teilobjekt. Damit dann die ganze Geometrie eines Objektes zusammen 
      is, muessen spaeter diese Linienobjekte zusammengesetzt werden."""

    def __init__(self, anfpkt, endpkt, artdergeo, paralist, folie, lineart):
        """bergeben wird ein Tupel der Daten wie bei setLiniendata und die
        Reihenfolge.  Die Reihenfolge gibt an ob die Reihenfolge der Punkte
        dieser Linie noch gendert werden muss.  Ist die Reihenfolge "R" ,
        dann wird die Reihenfolge in der die Pkte kommen gendert und dann
        setLiniendata aufgerufen.  Ist die Reihenfolge "N" wird nur
        setLiniendata aufgerufen und die Reihenfolge der Punkte nicht mehr
        gendert

        bergeben wird ein Liste mit folgenden Daten:
           *Anfangspkt
           *Endpkt
           *Art der Liniengeometrie
           *Lageparametreliste = [Pkt]
           *Folie
           *Linienart
        """

        self.Anfangspkt = anfpkt
        self.Endpkt = endpkt
        self.artdergeo = artdergeo
        self.paralist = paralist
        self.Folie = folie
        self.lineart = lineart

    def get_Vektor(self):
        """Macht aus den ersten zwei Punkten einen Linien-Vektor"""

        # Hole Anfangs- und Endpunkt, Endpunkt ist davon abhngig, ob ein 
        # Lageparameter vorhanden ist oder nicht
        anfPunkt = self.Anfangspkt
        if self.paralist == []:
            endPunkt = self.Endpkt
        else:
            endPunkt = self.paralist[0]

        # Berechnung des Vektors
        vektor = (endPunkt[0] - anfPunkt[0],
                endPunkt[1] - anfPunkt[1])

        return vektor 

    def winkel(self, vektor, Linie, winkelart):
        """Bei winkelart == 'I': ist der Innenwinkel von self kleiner als
        der von Linie bezglich des Vergleichsvektors?  Bei winkelart ==
        'A': ist der Aussenwinkel von self kleiner als der von Linie bzg
        Vergleichsvektor.  Annahme: der Vergleichsvektor ist schon in
        Koordinaten transformiert.
           """

        # Existiert keine Vergleichslinie ist man fertig
        if Linie == None:
            return 1

        # macht aus den letzten beiden Punkten der Linie self
        # einen Einheitsvektor    
        r1 = self.get_Vektor()        
        r1Length = sqrt(pow(r1[0],2) + pow(r1[1],2))
        r1Ein =(r1[0] /r1Length,r1[1] / r1Length)

        # macht aus den letzten beiden Punkten der Linie Linie
        # einen Einheitsvektor
        r2 = Linie.get_Vektor()
        r2Length = sqrt(pow(r2[0],2) + pow(r2[1],2))
        r2Ein =(r2[0] /r2Length,r2[1] / r2Length)

        # erzeugt die winkel zwischen Vergleichsvektor und den Einheitsvektoren
        angle_r1 = vektor[0] * r1Ein[0] + vektor[1] * r1Ein[1]
        angle_r2 = vektor[0] * r2Ein[0] + vektor[1] * r2Ein[1]

        # erzeugt den winkel zwischen um 90Grad verdrehten Vergleichsvektor
        # und den Einheitsvektoren
        orthoangle_r1 = vektor[1] * r1Ein[0] - vektor[0] * r1Ein[1]
        orthoangle_r2 = vektor[1] * r2Ein[0] - vektor[0] * r2Ein[1]

        # Je nach winkelart, die berprft werden soll, wird das Ergebnis
        # je Fall zurckgegeben
        if winkelart == "I":
            if orthoangle_r1 > 0 and orthoangle_r2 < 0:
                return 1
            elif orthoangle_r1 > 0 and angle_r1 <= angle_r2:
                return 1
            elif orthoangle_r1 == 0 and angle_r1 <= angle_r2:
                return 1
            elif orthoangle_r1 <= 0 and orthoangle_r2 <= 0 \
                and angle_r1 >= angle_r2:
                return 1
            else:
                return 0
        elif winkelart == "A":
            if orthoangle_r1 < 0 and orthoangle_r2 > 0:
                return 1
            elif orthoangle_r1 < 0 and angle_r1 <= angle_r2:
                return 1
            elif orthoangle_r1 == 0 and angle_r1 <= angle_r2:
                return 1
            elif orthoangle_r1 >= 0 and orthoangle_r2 >= 0 \
                and angle_r1 <= angle_r2:
                return 1
            else:
                return 0

# Folgende Klassen sind fr den Feature 
class Linienzug:
    """Diese Klasse ist ein Kontainer fr eine Liste von Punkten, die
    eine Linie darstellen."""

    def __init__(self):
        """Initalisiert die Punktliste"""
        self.pktlist =[]

    def vorneeinfuegen(self , Punkt):
        """fgt Punkt  vorne in die Punktliste ein"""
        self.pktlist = [ Punkt ] +self.pktlist

    def hinteneinfuegen(self, Punkt):
        """fgt Punkt hinten in die Punktliste ein"""
        self.pktlist.append(Punkt)

    def getAnfangspkt(self):
        """gibt den ersten Punkt der Punktliste zurck"""
        return self.pktlist[0]

    def getEndpkt(self):
        """gibt den letzten Punkt der Punktliste zurck"""
        return self.pktlist[-1]

    def getganzeListe(self):
        """gibt die ganz Punktliste zurck"""
        return self.pktlist

class Ring:
    """Kontainer fr die Punkte eines Ringes"""
    def __init__(self, list):
        """Initialisiert die Punktliste"""
        self.pktlist = list
    def einfuegen(self,Punkt):
        """fgt einen Punkt  hinten in die pktlist an"""
        self.pktlist.append(Punkt)
    def getEndpkt(self):
        """gibt den letzten Punkt der pktlist zurck"""
        return self.pktlist[-1]
    def complete(self):
        """schaut ob Anfangs- und Endpkt zusammenpassen"""
        return self.pktlist[0] == self.pktlist[-1] 
    def getganzeListe(self):
        """Gibt die ganze Punktliste zurck"""
        return self.pktlist
    def isPolygon(self):

        area = 0
        pktList = self.getganzeListe()
        (refx, refy) = pktList[0]
        for i in range(len(pktList)):
            (p1x, p1y) = pktList[i]
            (p2x, p2y) = pktList[i-1]
            area += (p1x - refx) * (p2y - refy) - (p2x - refx) * (p1y - refy)
        return area > 0


class Polygon:
    """Kontainer fr ein Polygon mit Lchern"""
    def __init__(self, ring):
        """Initialisiert die Punkt- und die Lcherliste"""
        self.pktlist = ring
        self.loecher = []
    def einfuegen(self, Punkt):
        """Fgt einen Punkt hinten in der pktlist ein"""
        self.pktlist.append(Punkt)
    def einfuegenLoch(self, Loch):
        """Fgt ein fertiges Loch in der Lochliste ein"""
        self.loecher.append(Loch)
    def getEndpkt(self):
        """gibt den letzten Punkt der Pktliste zurck"""
        return self.pktlist[-1]
    def getganzeListe(self):
        """gibt die ganzePunktliste zurck"""
        return self.pktlist
    def getLoecher(self):
        """gibt die Lcher zurck"""
        return self.loecher

class Loch:
    """Kontainer fr die Punkte eines Loches"""
    def __init__(self, ring):
        """Initialisiert die Punktliste"""
        self.pktlist = ring
    def einfuegen(self,Punkt):
        """fgt einen Punkt  hinten in die pktlist an"""
        self.pktlist.append(Punkt)
    def getEndpkt(self):
        """gibt den letzten Punkt der pktlist zurck"""
        return self.pktlist[-1]
    def getganzeListe(self):
        """Gibt die ganze Punktliste zurck"""
        return self.pktlist

class Feature:
    """Ein Objektdaten-Sammler speicher alle Information zu einem
    EDBS-Objekt, d.h. alle Linieninformationen, alle Besondereren
    Informationen und alle Teilobjekte.

    Ein Objekt hat ein Status, der angibt ob beim Eintragen
    das Objekt in der DB vorher gelscht werden soll , nur gelscht
    werden soll, oder verndert werden soll oder neu eingetragen.
    Folgende Ausprgungen hat das Statusbyte: 
          N: Neueintrag
          L: Objekt wird gelscht
          M: Objekt wird gelscht und danach neu eingetragen
          E: Objekt wird gendert

    Die gamma-Daten sind in self.II_daten in der Form : 
        (FunktiondesObjektes,[([BesondereInfos], [Geometrieangaben])])
    gespeichert.
    """

    def __init__(self, Objnr):
       """Konstruktor: Das Statusfeld wird auf N gesetzt
        und die Objnr gespeichert."""
       self.Status ="N"
       self.Objnr = Objnr
       self.linelist = []
       self.Objkoor = (0.0 , 0.0)
       self.AK = None
       self.folie = ''
       self.art = ''
       self.aktualitaet = ''
       self.typ = ''
       self.datum = ''
       self.tag = ''
       self.monat = ''
       self.jahr = ''
       self.objektteile = []
       self.BIAtt = []
       self.BIVal = []

    def compareObjnr(self , Objnr):
        """True, wenn 'Objnr' mit der Objektnr des Objektes identisch ist"""
        return self.Objnr == Objnr

    def appendline(self, line):
        """Fgt eine neue line-Objekt am Ende der Liste von Feature
        ein.  Falls das Statusfeld auf 'L', nderte das nach 'M' """
        self.linelist += [line]
        if self.Status == 'L':
           self.Status = 'M'
       

    def loescheObj(self,AK):
       """Setzt das Statusfeld auf 'L', lscht alle Daten in Feature"""

       if self.AK == None or self.AK == AK:
           self.Status = 'L'
           self.linelist = []
           self.AK = AK

    def setObjdaten(self, gamma, Objpunkt):
       """Setzt die Objektdaten mit Hilfe eines gamma-Teils des Objekt"""
       self.Objkoor = Objpunkt
       self.folie = gamma.objfkt[0:3]
       self.art = gamma.objfkt[3:7]
       self.aktualitaet = gamma.objfkt[7:9]
       self.typ = gamma.objfkt[9:10]
       self.jahr = gamma.objfkt[17:19]
       self.monat = gamma.objfkt[19:21]
       self.tag = gamma.objfkt[21:23]
       self.datum = gamma.objfkt[17:23]
       self.objektteile = self.extractOT(gamma.betalist)

       if self.Status =='L':
           self.Status = 'M'

    def extractOT(self, daten):

        otList = []
        for d in daten:
            ot = objektTeil()
            ot.setDaten(d)
            otList.append(ot)
        return otList

    def get_ganzeLinie(self):
        """Die einzelnen edbs_line-Objekte stellen, wenn das grosse Objekt
        eine Linie ist, nur Teilstcke dar. Diese Teilstcke werden von
        dieser Funktion in verschiedene Linienzge zusammengefasst.  Die
        Rckgabe ist eine Liste von Linienzug-Objekten"""

        # Hier werden die benutzten Datenstrukturen initialisiert
        # queue ist eine Warteschlange von edbs_line -Objekten
        # Diese Objekte sollen zu einer ganzen Linie zusammengesetzt werden.
        queue = self.linelist[:]
        # ergebnis beinhaltet die Linienzge, die aus den
        #einzelnen edbs_line-Linien zusammengesetzt sind
        ergebnis =[]
        count =0

        # Solange noch Linienteile zu verarbeiten sind, also der queue nicht
        # leer ist, gehe diese Ababeitungsschleife durch        
        while queue != []:

            # Das zu bearbeitende Linienstck wird aus den queue entnommen 
            linie = queue[0]
            anfpkt = linie.Anfangspkt
            endpkt = linie.Endpkt
            Art = linie.lineart
            del queue[0]

            # Eintrag der ersten Linie
            if ergebnis == []:
                # Am Anfang soll keine temporre Linie eingefgt werden
                if Art == '0242':
                    queue.append(linie)
                # der erste Lineinzug wird eingefgt
                else:
                    count =0
                    Zug = Linienzug() # Der erste Linienzug
                    Zug.hinteneinfuegen(anfpkt)
                    for pkt in linie.paralist:
                        Zug.hinteneinfuegen(pkt)
                    Zug.hinteneinfuegen(endpkt)
                    ergebnis = [ Zug ]

            # Linienstcke werden immer nur an die letzte Linie in der
            # Ergebnisliste eingefgt. Eine temporre Linie, die von einer
            # anderen Linie weggeht, beendet diese automatisch.
            else:
                ergebnisline = ergebnis[-1]
                # Endpunkt der letzten Linie und Anfangspkt der neuen
                # stimmen berein
                ergEndpkt = ergebnisline.getEndpkt()
                ergAnfpkt = ergebnisline.getAnfangspkt()
                if ergEndpkt == anfpkt or ergEndpkt == endpkt:
                    count = 0
                    # temporre Linie wird ignoriert, wenn sie zurck fhrt
                    # zur vorletzten Linie. Sonst fhrt sie zum Anlegen
                    # eines neuen Liniezuges.
                    if Art == "0242":
                        # Temporre Linie fhrt lediglich zurck zur
                        # letzten Linie
                        if len(ergebnis) >= 2 \
                            and (ergebnis[-2].getAnfangspkt() == anfpkt\
                            or ergebnis[-2].getAnfangspkt() == endpkt\
                            or ergebnis[-2].getEndpkt() == endpkt\
                            or ergebnis[-2].getEndpkt() == anfpkt):
                            pass
                        else:
                            Zug = Linienzug()
                            if ergEndpkt == anfpkt:
                                Zug.hinteneinfuegen(endpkt)
                            else:
                                Zug.hinteneinfuegen(anfpkt)
                            ergebnis += [ Zug ]
                    # anderen Linienstcke werden einfach an den letzten
                    # Liniezug angehngt
                    else:
                        tmp_liste = linie.paralist
                        lastpoint = endpkt
                        if ergEndpkt == endpkt:
                            tmp_liste.reverse()
                            lastpoint = anfpkt
                        for pkt in tmp_liste:
                            ergebnis[-1].hinteneinfuegen(pkt)
                        ergebnis[-1].hinteneinfuegen(lastpoint)
                elif ergAnfpkt == endpkt or ergAnfpkt == anfpkt:
                    count = 0
                    # temporre Linie wird ignoriert, wenn sie zurck fhrt
                    # zur vorletzten Linie. Sonst fhrt sie zum Anlegen
                    # eines neuen Liniezuges.
                    if Art == "0242":
                        # Temporre Linie fhrt lediglich zurck zur
                        # letzten Linie
                        if len(ergebnis) >= 2 \
                            and (ergebnis[-2].getAnfangspkt() == anfpkt\
                            or ergebnis[-2].getAnfangspkt() == endpkt\
                            or ergebnis[-2].getEndpkt() == endpkt\
                            or ergebnis[-2].getEndpkt() == anfpkt):
                            pass
                        else:
                            Zug = Linienzug()
                            if ergAnfpkt == endpkt:
                                Zug.hinteneinfuegen(anfpkt)
                            else:
                                Zug.hinteneinfuegen(endpkt)
                            ergebnis += [ Zug ]
                    else:
                        # Punkte werden in umgekehrter Reihenfolge am
                        # vorderen Ende der Linie eingefgt
                        tmp_liste =linie.paralist
                        firstpoint = anfpkt
                        if ergAnfpkt == anfpkt:
                            tmp_liste.reverse()
                            lastpoint = endpkt
                        for pkt in tmp_liste:
                            ergebnis[0].vorneeinfuegen(pkt)
                        ergebnis[-1].vorneeinfuegen(firstpoint)
                # Linie passt nicht, und wird wieder hinten an die queue
                # gehngt. Wenn alle Linienstcke bearbeitet wurden ohne zu
                # passen, wird abgebrochen.
                else:
                    if count < len(queue):
                        queue.append(linie)
                    else:
                        # TODO: wenn nur templinien brig sind, gibt es
                        # kein Problem
                        if queue != []:
                            log.warning("Warnung: Linienzug kann nicht vervollstndigt werden "+self.Objnr +"\n" )
                            count = 0
                            break
                    count +=1

        # Falls Anfangspkt des ersten Linienzugs und Endpkt des letzten
        # Linienzugs gleich sind, wird eine Warung ausgegeben
        if ergebnis[0].getAnfangspkt() == ergebnis[-1].getEndpkt():
            log.warning("Warnung: Der Anfangs- und Endpunkt der Linie fr Objekt %s sind identisch \n" % self.Objnr)

        return ergebnis

    def get_Polygon(self): 
        """Die einzelnen edbs_line-Objekte in einen Polygon stellen nur 
        Teilstcke dar. Diese Teilstcke werden hier zusammengefasst.
        Es kann auch vorkommen, dass ein Polygon Lcher hat. 
        Die Rckgabe ist eine Liste von Polygonen

        Der Algorithmus sollte folgendermassen sein:
        Man macht alles erst mit den Innenwinkel. Wenn das nicht klappt nochmal
        mit dem Aussenwinkel statt dem Innenwinkel:
        Zuerst baut man ein Polygon -Objekt auf und schreibt es in die
        Arbeitsliste.
        Man geht die noch nicht gearbeiteten Linienstcke durch und findet die 
        darzustellendes Liniestcke mit kleinsten Innenwinkel und
        die mglicherweise vorhandene nichtdarzustellende Liniestck, 
        die von den Endpkt losgeht.
        Jetzt gibt es drei Flle:
        1. Es gibt eine Nichtdarzustellendes -Linienstck  und beim letzten
        Durchlauf wurde keine Nichtdarzustellendes Linienstck bearbeitet.
        2. Es gibt ein darzustellendes Liniestck
        3. Es gibt keins von beiden: dann muss man zu den zweiten Grobdurchgang
          schalten(den mit Aussenwinkel),wenn man alle mal durchgegangen
          ist
        Der 1. Fall gliedert sich weiter, je nachdem auf der ringlist an
        0.er Stelle ein Loch oder ein Polygon ist.
        Ausserdem wird nun die nichtdarzustellende Linie bearbeitet und kann
        aus dem queue geloescht werden.   
        1.1. Ist auf der Arbeitsliste ein Loch:
        1.1.1 Ist das Loch fertig: ja. Dann wird das Loch in das oberste 
        Polygon geschrieben 
        1.1.2 Nein, dann wird ein neues Loch begonnen
        1.2 Ist auf der Arbeitsliste ein Polygon,
        dann wird gecheckt ob das Polygon fertig ist:
        1.2.1. Wenn ja, dann wird es in das ergebnis gepackt
        1.2.2. Wenn nein, wird je nach verhltnis des darzustellendes und des
        nichtdarzustellenden Linienstckes ein neues Loch bzw Polygon gebaut und
        auf die ringlist geschickt
        2. Fall: Es wird das Linienstck an das ersten Element der ringlist
        angebaut.   
        Falls weder
        """

        if Debug_Ordner:
            self.debug_export()
        # Der Algorithmus mus zweimal durchlaufen werden, wenn es 
        # mit den Innenwinkeln bei ersten Mal nicht klappt.
        # Beim zweiten mal benutzt man Aussenwinkel
        for buch in ["I","A"]: 

            queue = self.linelist[:]
            ringlist = []
            lochlist = []
            ergebnis = []
            count = 0

            # Zum erkennen ob vorher eine Nichtdarzustellende Linie
            # verarbeitet wurde 
            nodarflag = 0 

            # Trage die erste darzustellenden Linie als Linie in das
            # Gesamtpolygon (ringlist) ein
            for linie in queue:
                if linie.lineart != "0242":

                    ring = Ring([])
                    # Anfangspunkt
                    ring.einfuegen(linie.Anfangspkt)
                    # alle Lageparameter
                    for pkt in linie.paralist:
                        ring.einfuegen(pkt)
                    # Endpunkt
                    ring.einfuegen(linie.Endpkt)

                    # Polygon 
                    ringlist += [ ring ]
                    queue.remove(linie)

                    # Erzeuge den ersten Vergleichsvektor
                    temp_koord = ringlist[0].getganzeListe()
                    vektor =(temp_koord[-1][0] - temp_koord[-2][0] ,
                                         temp_koord[-1][1] - temp_koord[-2][1])
                if len(ringlist) > 0:
                    break

            while queue != [] and count < len(queue):

                if ringlist == []:
                    break 

                # Durchsuche noch nichtverarbeitete Linien nach
                # - passenden nichtdarzustellenden Linien
                # - passenden darzustellenden Linien, dort diejenige mit
                #   dem kleinsten Innenwinkel
                # Darzustellende linie mit mimimalen Innenwinkel
                darlinie= None
                # Nichtdarzustellende Linie
                nodarlinie = None
                for linie in queue:
                    anfPkt = linie.Anfangspkt
                    vEndPkt = ringlist[0].getEndpkt()
                    if anfPkt == vEndPkt:
                        if linie.lineart == "0242":
                            if nodarflag == 0:
                                nodarlinie = linie
                        elif linie.winkel(vektor, darlinie, buch):
                            darlinie = linie

                if nodarlinie != None:

                    endPkt = nodarlinie.Endpkt
                    anfPkt = nodarlinie.Anfangspkt
                    # nodarline springt zu altem Objekt zurck
                    if len(ringlist) > 1 \
                        and endPkt == ringlist[1].getEndpkt():
                        ring = ringlist[0]
                        if not ring.complete():
                            print "Fehler: Ring sollte vollstndig sein!"

                        # -> Polygon wird in Ergebnis eingetragen
                        elif ring.isPolygon():
                            polygon = Polygon(ring.getganzeListe())
                            for loch in lochlist:
                                polygon.einfuegenLoch(loch)
                            lochlist = []
                            ergebnis += [ polygon ]

                            del ringlist[0]
                        # -> Loch wird in aktuelles Polygon eingetragen
                        # TODO: berprfen ob nodarline nicht auch 
                        # zwei Lcher verbinden kann
                        else:
                            loch = Loch(ring.getganzeListe())
                            del ringlist[0]
                            lochlist.append(loch)

                    # nodarline springt nicht zurck
                    # -> ein neuer Ring wird begonnen
                    else:
                        neuerRing = Ring( [ endPkt ])
                        ringlist = [ neuerRing ] + ringlist
                        nodarflag = 1

                    # neuen Vergleichsvektor bestimmen
                    anfPkt = nodarlinie.Anfangspkt
                    endPkt = nodarlinie.Endpkt
                    lagepara = nodarlinie.paralist
                    if lagepara == []:
                        vektor =(endPkt[0] - anfPkt[0], endPkt[1] - anfPkt[1])
                    else:
                        lastPkt = lagepara[-1]
                        vektor =(endPkt[0] - lastPkt[0], endPkt[1] - lastPkt[1])

                    count = 0
                    queue.remove(nodarlinie)

                elif darlinie != None:

                    # neuen Punkt eintragen
                    anfPkt = darlinie.Anfangspkt
                    endPkt = darlinie.Endpkt
                    lagepara = darlinie.paralist
                    for pkt in lagepara:
                        ringlist[0].einfuegen(pkt)
                    ringlist[0].einfuegen(endPkt)

                    # neuen Vergleichsvektor bestimmen
                    if lagepara == []:
                        vektor =(endPkt[0] - anfPkt[0], endPkt[1] - anfPkt[1])
                    else:
                        lastPkt = lagepara[-1]
                        vektor =(endPkt[0] - lastPkt[0], endPkt[1] - lastPkt[1])

                    count = 0
                    queue.remove(darlinie)
                    nodarflag = 0
                else:
                    count += 1
                    if count > len(queue):
                        log.warning("warning: Es muss mindestens eine darlinie -existieren")
                        break

            # Wenn der Algorithmus mit kleinstem Innenwinkel erfolgreich
            # war (d.h. alle Linienelemente sind verarbeitet, wird
            # abgebrochen.
            if queue == []:
                for ring in ringlist:
                    if ring.isPolygon():
                        polygon = Polygon(ring.getganzeListe())
                        for loch in lochlist:
                            polygon.einfuegenLoch(loch)
                        ergebnis += [ polygon ]
                    del ring
                return ergebnis

        return ergebnis

    def get_RahmenobjektPolygon(self):
        RECT_LENGTH = 62.5
        
        (x, y) = self.Objkoor
        coords = [(x, y),
                  (x, y + RECT_LENGTH),
                  (x + RECT_LENGTH, y + RECT_LENGTH),
                  (x + RECT_LENGTH, y), 
                  (x, y)]

        return Polygon(coords)

    def debug_export(self):
        import ogr
        import os
        folder = Debug_Ordner + os.sep

        # Die Daten sollen als Shapefile qausgegeben werden.
        driver = ogr.GetDriverByName("ESRI Shapefile")
        
        # Fr jede Objnr soll eine eigene Datei angelegt werden.
        # TODO Verzeichnisnamen angeben.
        ds = driver.Open(
                folder + self.Objnr +".shp" , 1)
        if ds == None:
            ds = driver.CreateDataSource(
                    folder + self.Objnr +".shp")
        layer = ds.CreateLayer(self.Objnr)

        layer.CreateField(ogr.FieldDefn('objnr', ogr.OFTString))
        layer.CreateField(ogr.FieldDefn('linetype', ogr.OFTString))

        counter = 0
        for line in self.linelist:
            points =  self.getPointsFromEDBS_Satz(line)
            linestr = self.getLineGeomFromPoints(points)
            geom = ogr.CreateGeometryFromWkt(linestr)
            
            feature = ogr.Feature(layer.GetLayerDefn())
            
            idx_objnr = feature.GetFieldIndex('objnr')
            feature.SetField(idx_objnr, self.Objnr + "-" + str(counter))
            
            idx_linetype = feature.GetFieldIndex('linetype')
            feature.SetField(idx_linetype, line.lineart)

            feature.SetGeometry(geom)
            
            layer.CreateFeature(feature)
            counter = counter + 1

        # Aufrumen
        ds.Destroy()

    def getPointsFromEDBS_Satz(self, line):
        points = [line.Anfangspkt]
        for pt in line.paralist:
            points.append(pt)
        points.append(line.Endpkt)
        return points

    def getLineGeomFromPoints(self, points):
        coords = []
        for (x,y) in points:
            coords.append('%s %s' %(round(x,3),round(y,3)))
        return 'LINESTRING(%s)' % string.join(coords,',')

class objektTeil:

    def __init__(self):

        self.artderInfo = ''
        self.kartentyp = ''
        self.darstellung = ''
        self.BI = ''
        self.artderGeo = ''
        self.geoangabenListe = []

    def setDaten(self, daten):

        self.artderInfo = daten[0][0][0:2]
        self.kartentyp = daten[0][0][2:4]
        self.darstellung = daten[0][0][6:10]
        self.BI = daten[0][0][10:43]
        self.artderGeo = daten[0][0][43:45]

        self.geoangabenListe = daten[1]

    def geoangaben(self):

        """Die Geometrie-angaben werden je nach art(Art der Geometrie aus
        den besonderen Infos) bearbeitet:
        11,15: Die Linienpkte werden rausgelesen:
                  1. Pfeilspitze 
                  2. Pfeilanfang 
                  3. Knickpkte
               und in richtige Reihenfolge und Format gebracht.

        51,52: Ein Pkt wird rausgelesen
        53: Ein Winkel und ein Pkt wird gelesen """

        if self.artderGeo in ('11', '15'):
            Linie =[]
            Linie += [punkttrans(self.geoangabenListe[0],'alk')]
            for n in range(2,len(self.geoangabenListe)):
                Linie += [punkttrans(self.geoangabenListe[n], 'alk')]
            Linie += [punkttrans(self.geoangabenListe[1],'alk')]
            return Linie

        elif self.artderGeo in ('51', '52'):
            return punkttrans(self.geoangabenListe[0], 'alk')

        elif self.artderGeo == '53':
            return [float(self.geoangabenListe[0][2:7])*9/1000, \
                punkttrans(self.geoangabenListe[1],'alk')]
        else:
           raise ValueError("error : die Art der Geometrieangeben ist unbekannt")

    def extractAttributes(self, attrSpec, table):
        """Holt aus den Besonderen Informationen eine Liste von Attributen
        und Werten entsprechend der Slices, die in attrSpec definiert
        sind"""

        SQL_Att = []
        SQL_Val = []
        for attr in attrSpec:
            if attr == '':
                break
            aname, aslice = attr.split(',')
            lowslice, highslice = map(int, aslice.split(':'))
            if lowslice > highslice or highslice > 33:
                raise "Fehler: unglitige Slice-Angaben fr \
                    Attribut %s in Tabelle %s" %(aname, table)
            else:
                SQL_Att.append(aname)
                rawvalue = self.BI[lowslice-1:highslice]
                SQL_Val.append("%s" % rawvalue)
        return SQL_Att, SQL_Val


class Alpha:
    """ Speichert Alpha-Daten eines EDBS-Satzes"""

    def __init__(self):

        self.endpoi = ''
        self.linetype = ''
        self.fktlinelist = []
        self.paralist = []

class Gamma:
    """ Speichert Gamma-Daten eines EDBS-Satzes"""

    def __init__(self):

        self.objfkt = ''
        self.betalist = []

class II_data:
    """Speichert Objektkoordinate, Alpha-Daten und Gamma-Daten eines
    EDBS-Satzes zurck"""

    def __init__(self):
        self.objkoor = ''
        self.alphalist = []
        self.gammalist = []


class edbs_satz:
    """ein Kontainer fr eine EDBS-Daten-Zeile
       Diese Klasse wird fr first_parsing(file) genutzt.Sie speichert eine
       Zeile eines EDBS-Datensatzes als self.EDBS_line und einen Wert
       self.pos der dazuda ist die II- inforamtionen rauszulesen mit der
       Methode get_II_data()"""

    def __init__(self, data):
        """Konstruktor von entry: Man bergibt eine EDBS-Zeile als String.
           Wandelt diese Zeile gemss translation in richtigen Zeichencode
           Setzt die pos auf 0"""

        self.EDBS_line = data.translate(translation)
        self.pos =0

    def __str__(self):
        """Wenn das Objekt zum String werden soll,
        gibt es den EDBS-Satz zurck, 
        der unter self.EDBS_line gespeichert ist."""
        return self.EDBS_line

    def relation(self):
        """gibt das Zugehrigkeitsbyte zurck"""
        return self.EDBS_line[22]

    def read_WHF(self):
        """holt sich 4 Bytes von der Position self.pos von self.EDBS_line,
        wandelt die in int um und gibt er zurck, erhht self.pos um 4  """
        tmp= int(self.EDBS_line[self.pos:self.pos+4])
        self.pos +=4
        return tmp

    def read_Point(self, edbsformat):
        """gibt 20 Bytes von Position self.pos zurck und erhht self.pos 
        um 20""" 

        tmp = self.EDBS_line[self.pos:self.pos+20]
        self.pos +=20
        return punkttrans(tmp, edbsformat)

    def read_Fktofline(self, edbsformat):
        """gibt 23 Bytes von Position self.pos zurck und erhht self.pos 
        um 23"""        
        if edbsformat == "atkis" or edbsformat == "nds_alk":
            cur = 29
        else:
            cur = 23
        tmp = self.EDBS_line[self.pos:self.pos+cur] 
        self.pos +=cur
        return tmp        

    def read_postionparameter(self):
        """gibt 20 Bytes von Position self.pos zurck und erhht self.pos 
        um 20"""
        tmp = self.EDBS_line[self.pos: self.pos +20 ] 
        self.pos +=20
        return tmp      

    def read_specialinfo(self, edbsformat):
        """gibt 45 Bytes von Position self.pos zurck und erhht self.pos 
        um 45"""
        if edbsformat == 'atkis' or edbsformat == 'nds_alk':
            cur = 48
        else:
            cur = 45
        tmp = self.EDBS_line[self.pos: self.pos + cur ] 
        self.pos += cur
        return tmp      

    def read_geometrie(self):
        """gibt 20 Bytes von Position self.pos zurck und erhht self.pos 
        um 20"""
        tmp = self.EDBS_line[self.pos: self.pos +20 ] 
        self.pos +=20
        return tmp                      

    def read_Function(self, edbsformat):
        """gibt 23 Bytes von Position self.pos zurck und erhht self.pos 
        um 23"""
        if edbsformat == 'atkis' or edbsformat == 'nds_alk':
            cur = 26
        else:
            cur = 23
        tmp = self.EDBS_line[self.pos: self.pos + cur ] 
        self.pos += cur
        return tmp

    def read_Fachparam(self):
        """gibt 9 Bytes von Position self.pos zurck und erhht self.pos 
        um 9"""
        tmp = self.EDBS_line[self.pos: self.pos +9 ] 
        self.pos +=9
        return tmp

    def get_II(self):
        """gibt self.EDBS_line ab 36 Byte zurck"""
        return self.EDBS_line[36:]      

    def get_II_data(self, edbsformat):

        """returns a list of IN(Infoname) datas:Objkoord,EndpktderLine
        FunktionderLine,Lagepara,FunktiondesObjektes,BesondersInfo,
        Geometrieangabe Dabei wird der II-Teil des Strings self.EDBS_line
        durchgegangen. 
        Rckgabe: Ein Objekt vom Typ II_data
           """
           
        II_daten = II_data()

        # An Position 36 beginnt der relevante II-Abschnitt des EDBS-Satzes
        self.pos = 36

        #first WHF==0001 normally
        if self.read_WHF() != 1: 
            raise  ValueError("error: WHF 1 at the beginning excepted")

        #second WHF_A ==0001 normally
        if self.read_WHF() != 1: 
            raise  ValueError("error: 2nd WHF not 1")

        II_daten.objkoor = self.read_Point(edbsformat)

        # Remember the "Prfzeichen"
        self.pos += 1 

        # Alphainformation: Lagaparameter

        # WHFalpha: Gibt an wie viele Lageinformationen in dieser EDBS-Satz
        # vorhanden sind
        whfalpha = self.read_WHF() 

        for n in range(whfalpha):                    

            # Neuen Datensatz anlegen
            ainfo = Alpha()
            if self.read_WHF() != 1:  #WHF_Bi == 1 normally
                raise ValueError("error: WHF in alpha cycle not 1")

            # gets "Endpunkt der Linie"
            ainfo.endpoi = self.read_Point(edbsformat) 

            # Art der Liniengeometrie
            # 11 Gerade 
            # 15 Polygonzug 
            # 21 Kreisbogen, rechtsdrehend 
            # 22 Kreisbogen, linksdrehend 
            # 41 Interpolierte Kurve (nicht Folie 001 bis 003) 
            # 51 Punkte in der Standardausrichtung, rechtwinklig zum Gitternetz.
            # 52 Reprsentative Punkte in Standardausrichtung.
            # 53 Punkte in besonderer Ausrichtung (mit Winkel). 
            ainfo.linetype = self.EDBS_line[self.pos: self.pos+2]
            self.pos += 2

            # Funktion der Linie (= Zugehrigekeit zu links- und
            # rechtsseitigen Objekten) (+ Fachparameter bei ATKIS)
            # whfTheta: Gibt an wie viele Links- und rechtsseitige
            # Objektzugehrigkeiten jetzt folgen.
            whftheta = self.read_WHF()
            ainfo.fktlinelist = []
            for n2 in range(whftheta):

                # Fachparameter fr ATKIS...
                if edbsformat == 'atkis' or edbsformat == 'nds_alk':
                    WHFCij = self.read_WHF()
                    if WHFCij < 1:
                        raise  "error: WHF in theta cycle not 1: " + str(self.pos)
                    else:
                        for whfc in range(WHFCij): 
                            fktline = self.read_Fktofline(edbsformat)
                            ainfo.fktlinelist += [fktline]
                    for whfh in range(self.read_WHF()): 
                        fachparam = self.read_Fachparam()

                # ...sind nicht relevant bei ALK
                else:
                    fktline = self.read_Fktofline(edbsformat)
                    ainfo.fktlinelist += [fktline ]

            # Lageparameter auslesen
            whfDi = self.read_WHF() # Anzahl der Lageparameter
            # TODO: Spline
            if ainfo.linetype == "41":
                if whfDi > 0:
                    whfDi -= 1
                    self.pos += 20

            # Auslesen der Parameter
            for n3 in range(whfDi):
                para = self.read_Point(edbsformat) 
                ainfo.paralist += [para]

            # Interpolation der Kreisbgen:
            # rechtdrehend
            if ainfo.linetype == '21':
                newparalist = arc2wkt(ainfo.endpoi, 
                        II_daten.objkoor,
                        ainfo.paralist[0])
                ainfo.paralist = newparalist
                ainfo.paralist.reverse()


            # linksdrehend
            if ainfo.linetype == '22':
                newparalist = arc2wkt(II_daten.objkoor, \
                        ainfo.endpoi,
                        ainfo.paralist[0])
                ainfo.paralist = newparalist

            # Schlielich einfgen der Alpha-Informationen in die Liste
            II_daten.alphalist.append(ainfo)

        # Gammainformation: Angaben ber Folie sowie
        # Objektteil-Informationen etc.
        whfgamma= self.read_WHF()  # WHFgamma
        for n3 in range(whfgamma):
            if self.read_WHF() != 1 :
                raise ValueError("error: gamma cylce WHF not 1")
            cinfo = Gamma()

            # Objektfunktion: Objekt-spezifische Sachdaten (Objnr etc.)
            cinfo.objfkt = self.read_Function(edbsformat)

            # Besondere Informationen (oft Objektteil-Informationen)
            whfbeta = self.read_WHF()
            for n4 in range(whfbeta):

                # Besondere Informationen
                whfFij = self.read_WHF()
                specialinfolist = []
                for n in range(whfFij):
                    specialinfolist += [ self.read_specialinfo(edbsformat) ]
                whfGij = self.read_WHF()

                # Geometrie der Besonderen Info
                geometrielist =[]
                for n2 in range(whfGij):           
                    geometrielist += [ self.read_geometrie() ]
                cinfo.betalist.append((specialinfolist, geometrielist))
            II_daten.gammalist.append(cinfo)

        return II_daten

    def get_entryart(self):
        """Je nach OP und IN gibt er folgendes zurck:
        OP    | IN       |Rckgabe
        BSPE  |ULOBNN..  |OBJEKTDATEN
        FEIN  |ULOBNN..  |OBJEKTDATEN
        BINF  |ULOTEX..  |AUFTRAGSKENNDATEN
        OTEX  |ILOTEX..  |AUFTRAGSTEXTDATEN
        FGEO  |ULODATUM  |AENDERUNGSDATUM  (fr BZSN)
        FLOE  |DLOB2005  |OBJDATENLOESCHEN (fr BZSN)
        BKRT  |IBENKRT.  |GEBIETSKENNZEICHNUNG
        AKND  |ULQA0000  |AUFTRAGSKENNSATZ
        AEND  |........  |AUFTRAGSENDE
        sonst            |UNDEFINIERT"""

        OP = self.EDBS_line[12:16]
        IN = self.EDBS_line[28:36]
        if  (OP == "BSPE" or OP == "FEIN") and IN=="ULOBNN  ":
            return "OBJEKTDATEN"
        elif(OP == "BSPE" or OP == "FEIN") and IN == "ULO8ALK ": 
            return "OBJEKTDATEN"
        elif OP == "BINF" and IN == "ULOTEX  ":
            return "AUFTRAGSKENNDATEN"
        elif OP == "OTEX" and IN == "ILOTEX  ":
            return "AUFTRAGSTEXTDATEN"
        elif OP == "FGEO" and IN == "ULODATUM":
            return "AENDERUNGSDATUM"
        elif OP == "FLOE" and IN == "DLOB2005":
            return "OBJDATENLOESCHEN"
        elif OP == "BKRT" and IN == "IBENKRT ":
            return "GEBIETSKENNZEICHNUNG"
        elif OP == "AKND" and IN == "ULQA0000":
            return "AUFTRAGSKENNSATZ"
        elif OP == "AEND" and IN == "        ":
            return "AUFTRAGSENDE"
        else:
            return "UNDEFINIERT"

    def get_SK_Objnr(self):
        """parst das SK(Suchkriterium) des entry -Objektes und gibt eine
        Objnr und eine Aktualitten zurck
        """

        if self.EDBS_line[36:48] == 'ASDLOB2005EQ' \
                and self.EDBS_line[55:67] == 'ANDLOB2003EQ':
            return (self.EDBS_line[48:55], self.EDBS_line[67:69])
        elif self.EDBS_line[36:48] == 'ASDLOB2003EQ' \
                and self.EDBS_line[50:62] == 'ANDLOB2005EQ':
            return (self.EDBS_line[62:69], self.EDBS_line[48:50])
        else:
           print(self.EDBS_line[48:53], self.EDBS_line[63:65])
           raise ValueError("error: Nicht-Verstandene SK-Zeile")    


class edbs_satz_liste   :
     """container for edbs_satz objects which belong together"""

     def __init__(self,data):
         """constructor of edbs_satz_liste, needs an entry"""
         self.list = [data]

     def additionalentry(self, data):
         """appends ein entry Objekt an die Liste self.list"""
         self.list += [data]

     def get_entryart(self):
         """geht die Liste von entry self.list noch OP und IN durch.
         schaut ob die alle gleich sind, sonst gibt er UNDEFINIERT zurck"""

         art=self.list[0].get_entryart()
         for en in self.list:
             if en.get_entryart() != art:
                 print "edbs_satz_art: "+ art +" self.list_art: "+"en.getentryart()"+en.get_entryart() 
                 return "UNDEFINIERT"
         return art

     def get_II_data(self, edbsformat):
         """nimmt alle entry in self.list und verbindet ihr II Teil zu einen
         grossen II-Teil der in ein neuen entry verarbeitet werden kann"""

         tmp_zusammenII =""
         for ein in self.list:
            Laenge = int(str(ein)[4:8])
            tmp_zusammenII += ein.get_II()[0:(Laenge -24)]
            # Dies scheint zu klappen. Kann aber Fehler machen:             
            # tmp_zusammenII += ein.get_II().rstrip()               
         self.zusammenIIentry = edbs_satz(str(self.list[0])[0:36]+tmp_zusammenII)

         return self.zusammenIIentry.get_II_data(edbsformat)

     def __str__(self):
         """verbindet die str-Ausgabe von den verschiedenen entry Objekten
            aus self.list und gibt dies Verbundene zurck"""

         output =""
         for x in self.list:
             output += str(x)
         return output


def getwinkel(v):
    """ Berechnet den Winkel des Vektors zur X-Achse in Anteilen von PI """

    from math import asin, acos, pi
    (vx, vy) = v
    if vx >= 0 and vy >= 0:
        return acos(vx)
    if vx >= 0 and vy < 0:
        return 2 * pi + asin(vy)
    if vx < 0 and vy >= 0:
        return acos(vx)
    if vx < 0 and vy < 0:
        return pi - asin(vy)

def arc2wkt(a, b, m):
    """ Berechnet linksdrehend das Kreissegement von a nach b um den
    Mittelpunkt m"""

    from math import asin, sin, cos, pi

    (mx, my) = m
    (ax, ay) = (a[0]-mx, a[1]-my)
    (bx, by) = (b[0]-mx, b[1]-my)
    r = sqrt(pow(ax, 2) + pow(ay, 2))
    r2 = sqrt(pow(bx, 2) + pow(by, 2))

    # Genauigkeit
    h = 10
    while h > r: h /= 10.0
    gamma = asin(sqrt(2 * h * r - pow(h, 2)) / r)

    alpha = getwinkel((ax/r, ay/r))
    beta = getwinkel((bx/r2, by/r2))

    clist = []
    if alpha < beta:
        delta = alpha + 2 * pi - gamma
    else:
        delta = alpha - gamma
    while delta > beta:

        clist.append((r*cos(delta)+mx, r*sin(delta)+my))
        delta -= gamma

    return clist

