# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2004 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

import os
import os.path
import logging
from string import join
log = logging.getLogger('')


from edbsclasses import Feature


class FeatureCollection:
    """Sammlung von Feature-Objekten sowie Export-Informationen. Objekt
    kann aufgerufen werden um eine Reihe von Features zu exportieren."""

    def __init__(self):

        self.datasource = ''
        self.deletelist = []
        self.tableconf = None
        self.ONS = {}

    def export(self):
        """ Erhlt einen ONS und eine konkrete Tabellen-Spezifikation sowie
        einen Verzeichnis-Pfad in den die SQL-Dateien abgelegt werden
        sollen"""

        # Fr alle Objnr die bekannt, werden die INSERT-Befehle erezugt
        print "Trage %d Objekte ein" % len(self.ONS)
        for Objnr in self.ONS.keys():
            obj = self.ONS[Objnr]
            if not obj.compareObjnr(Objnr):
                print Objnr , self.ONS[Objnr].compareObjnr(Objnr)  
                raise ValueError("error: Ein objdaten_sammler hat andere Objnr als im ONS steht")

            # Zuerst werden die Lschoperationen bearbeitet
            if obj.Status in ['L','M']:
                self.deletelist.append(obj.Objnr)

            # Danach werden die INSERT-operationen bearbeitet
            if obj.Status in ['M','N']:
                SQL_Daten = exportFeature(self.ONS[Objnr], self.tableconf)
                sqlfile = open(os.path.join(self.datasource, 'f%s.sql' \
                    % obj.folie), 'a')
                sqlfile.write(SQL_Daten)
                sqlfile.close()

        if len(self.deletelist) > 0:
            print "Lsche %d Objekte ein" % len(self.deletelist)
            self.delete()

        print "Neubau der Indizes"
        self.rebuild_index()


    def delete(self):
        """Diese Funktion geht die ONS durch und sammlet alle Objnr mit
        Lschkommandos. Dann werden die DELETE-Kommados fr die einzelnen
        Tabellen geschrieben"""

        sqlfile = open(os.path.join(self.datasource, 'delete.sql'), 'a')
        # Erzeugen der SQL-Kommandos
        if self.deletelist != []:
            # Geht die Objnr der zu loschenden Eintrge durch
            # und Teilt sie in 225 Gruppen.
            SQL_Objnrn_liste =[]
            while self.deletelist:
                jetzt_loeschen = ["'%s'" % obj for obj in self.deletelist[:220]]
                self.deletelist = self.deletelist[220:]
                SQL_Objnrn_liste.append(', '.join(jetzt_loeschen))

            # Geht die Tabellen durch und schreibt fr jeden SQL_Objnrn-String
            # einen SQL-Befehl
            for tab in self.tableconf.tabnamen:
                if tab:
                    for SQL_Objnrn in SQL_Objnrn_liste:
                        sqlfile.write("DELETE FROM %s WHERE objnr IN (%s); \n"
                        % (tab, SQL_Objnrn) )


    def createSchema(self):
        """ Erzeugt Schemadefinitionen als SQL-Kommandos und schreibt sie
        in das sqlfile"""

        sqlfile = open(os.path.join(self.datasource, 'createalk.sql'), 'a')
        self.createSQL(sqlfile)

    def createTrigger(self, sqlfile, tabname):

        seqSQL = """
CREATE SEQUENCE %s_seq
MINVALUE 1
MAXVALUE 999999999999999999999999999
START WITH 1
INCREMENT BY 1
CACHE 20; """ % tabname

        triggerSQL = """
create trigger %(name)s_trigger
before insert on %(name)s
for each row
begin
    select %(name)s_seq.nextval into :new.ogr_fid from dual;
end;
/ """ %{'name': tabname}


        sqlfile.write(seqSQL)
        sqlfile.write(triggerSQL)

    def createMeta(self, sqlfile, tabname):

        metadatenSQL = """
INSERT INTO USER_SDO_GEOM_METADATA VALUES('%(name)s',
'geom',
MDSYS.SDO_DIM_ARRAY(
MDSYS.SDO_DIM_ELEMENT( 'X', 3345000, 3440000, 0.001 ),
MDSYS.SDO_DIM_ELEMENT( 'Y', 5682322, 5754034, 0.001 )
), NULL);
CREATE INDEX %(name)s_idx ON %(name)s (geom)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;
""" % {'name':tabname}

        sqlfile.write(metadatenSQL)

    def createSQL(self, sqlfile):

    
        newtables = {}
        geotables = []
        isobjtable = 0
        tables = self.tableconf.tables
        for tablekey in tables.keys():

            for tablename in tables[tablekey].keys():

                curTableConf = tables[tablekey][tablename]
                if tablename == '':
                    isobjtable = 1 
                    curtablekey = (tablekey[0],'','')
                    objTableConf = tables[(curtablekey[0],'','')]
                    tablename = objTableConf.keys()[0]
                    tabletypes = objTableConf[tablename]['types']
                elif tablekey == (tablekey[0],'',''):
                    isobjtable = 1
                    tabletypes = curTableConf['types']
                else:
                    isobjtable = 0
                    tabletypes = curTableConf['types']

                for typ in tabletypes:

                    if typ == 'n':
                        fulltablename = tablename
                    else:
                        fulltablename = "%s_%s" % (tablename, typ)
                        if fulltablename not in geotables:
                            geotables.append(fulltablename)

                    if fulltablename not in newtables.keys():

                        newattr = ["objnr VARCHAR(7)"]
                        newattr.append("ogr_fid INTEGER")
                        # Objekt-Tabelle
                        if isobjtable == 1:

                            newattr.append("objart INTEGER")
                            newattr.append("aktualitaet VARCHAR(2)")
                            newattr.append("edatum DATE")
                            newattr.append("geom MDSYS.SDO_GEOMETRY")
                            if typ == 'p':
                                newattr.append("winkel INTEGER")
                        # Objektteil-Tabelle
                        else:
                            newattr.append("artderinfo INTEGER")
                            newattr.append("kartentyp VARCHAR(2)")
                            newattr.append("darstellungsart INTEGER")
                            if typ in ('p','l','f'):
                                newattr.append("geom MDSYS.SDO_GEOMETRY")
                            if typ == 'p':
                                newattr.append("winkel INTEGER")
                        newtables[fulltablename] = newattr

                    attributes = curTableConf['attributes']
                    if attributes != '':
                        for attr in attributes.split(';'):
                            attrname, attrslice = attr.split(',')
                            lowslice, highslice = attrslice.split(':')
                            attrspec = "%s VARCHAR(%i)" \
                                % (attrname, int(highslice) - int(lowslice) + 1)
                            if attrspec not in newtables[fulltablename]:
                                newtables[fulltablename].append(attrspec)

        for t in newtables.keys():

            sqlfile.write("CREATE TABLE %s\n(%s);\n\n"\
                %(t, join(newtables[t], ',\n')))

        for t in geotables:
            self.createTrigger(sqlfile, t)
            self.createMeta(sqlfile, t)
        sqlfile.write("exit\n")


    def dropSchema(self):

        sqlfile = open(os.path.join(self.datasource, 'drop.sql'), 'a')
        oldtables = []
        geotables = []
        tables = self.tableconf.tables
        for tablekey in tables.keys():

            for tablename in tables[tablekey].keys():

                if tablename == '':
                    continue

                tabletypes = tables[tablekey][tablename]['types']
                for typ in tabletypes:

                    if typ == 'n':
                        fulltablename = tablename
                    else:
                        fulltablename = "%s_%s" % (tablename, typ)
                        if fulltablename not in geotables:
                            geotables.append(fulltablename)

                    if fulltablename not in oldtables:
                        oldtables.append(fulltablename)

        for t in oldtables:

            sqlfile.write("DROP TABLE %s;\n" %t)

        for t in geotables:
            
            sqlfile.write("DROP TRIGGER %s_trigger;\n" % t)
            sqlfile.write("DROP SEQUENCE %s_seq;\n" % t)
            sqlfile.write("DELETE FROM USER_SDO_GEOM_METADATA\
                WHERE TABLE_NAME = '%s';\n" % t.upper())
        sqlfile.write("exit\n")
          
    
    def rebuild_index(self):
        """Liest die Liste der geometrischen Tabellen aus
        self.tableconf.tabnamen und schreibt ein ALTER INDEX-Kommando in
        die Datei index.sql"""

        sqlfile =  open(os.path.join(self.datasource, 'index.sql'), 'w')
        # Erzeuge die Alter Index Rebuild -SQL-Kommandos jetzt hier
        for tabname in self.tableconf.tabnamen:
            if tabname[-2:] in ['_p', '_l', '_f' ]:
                sqlfile.write("ALTER INDEX %s_idx REBUILD;\n" % tabname)
        sqlfile.close()


def exportGeo(objektteil):
    """Erzeugt ein SQL-Kommando (DECLARE) zur Definition der Geometrie
    eines Objektteils (Punkt oder Linie, ggf. mit Winkel)"""

    winkel = None

    # Linie
    if objektteil.artderGeo in ['11', '15']:
        linie = objektteil.geoangaben()
        linelist = []
        coord_anzahl = 0 
        # nach 30 coord-Paare ein newline einfgen
        # Alle Coordinaten eintragen...
        for (x,y) in linie:
            if coord_anzahl == 30:
                endline = '\n'
                coord_anzahl = 0
            else:
                endline = ''
            linelist.append('%s,%s%s' %(round(x,3),round(y,3), endline))
            coord_anzahl += 1
        #... und zusammenfgen
        linienstr = join(linelist,',')

        declareSQL = "\
DECLARE\ngeom mdsys.sdo_geometry :=\n\
MDSYS.SDO_GEOMETRY(\
2002, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(1, 2, 1),\n\
MDSYS.SDO_ORDINATE_ARRAY(%s));\n" % linienstr

    # Punkt
    else:
        # ohne Winkelangabe
        if objektteil.artderGeo in ('51', '52'):
            (punkt_x, punkt_y) = objektteil.geoangaben()

        # mit Winkelangabe
        elif objektteil.artderGeo == '53':
            (winkel,(punkt_x, punkt_y)) = objektteil.geoangaben()
        declareSQL = "\
DECLARE\ngeom mdsys.sdo_geometry :=\n\
MDSYS.SDO_GEOMETRY(2001, null, \
MDSYS.SDO_POINT_TYPE(%s,%s,null), null, null);\n"\
            % (str(round(punkt_x,3)), str(round(punkt_y,3)))

    return (declareSQL, winkel)

def anpassen(Text):
    """Passt den Text so an, dass in Text ' richtig in die DB kommt.
       Also in einen String Text wir das einfache Anfuehrungszeichen
       ' verdoppelt und sorgt dafr, dass die hinternen Leerzeichen
       verschwinden."""           
    Text_angepasst = ""
    for ch in Text:
        if ch == "'":
            Text_angepasst += "''"
        else:                  
            Text_angepasst += ch
    Text_angepasst= Text_angepasst.rstrip()
    return "'%s'" % Text_angepasst


def exportObjektteil(obj, objektteil, attrSpec, table):
    """Exportiert ein Objektteil eines bestimmten Objektes"""

    SQL_Kom = "" 
    # Objektteil-spezifische Attribute
    SQL_Att = ['objnr', 'kartentyp', 'darstellungsart']
    SQL_Val = ["'%s'" % obj.Objnr, \
        "'%s'" % objektteil.kartentyp, \
        '%s' % objektteil.darstellung]

    # Attribute aus den besonderen Informationen
    SQL_Att_tmp, SQL_Val_tmp = \
        objektteil.extractAttributes(attrSpec,table)
    # anpassen der Werte fr den Eintrag in eine Oracle-DB
    SQL_Val_tmp = map(anpassen, SQL_Val_tmp)

    SQL_Att += SQL_Att_tmp
    SQL_Val += SQL_Val_tmp

    # SQL-DECLARE fr die Geodaten, wenn vorhanden, wird auch ein Winkel
    # erzeugt
    objtyp = ''
    # Schaut die Geodaten durch
    if objektteil.artderGeo in ['51','52','53', '11', '15']:
        (SQL_Kom, winkel) = exportGeo(objektteil)

        # Geometry -Attribut
        SQL_Att.append('geom')
        SQL_Val.append('geom')
        if objektteil.artderGeo in ['51','52','53']:
            objtyp = "_p"
            if objektteil.artderGeo == '53':
                SQL_Att.append('winkel')
                SQL_Val.append('%d' % winkel)

        else:
            objtyp = "_l"

    # Erzeugen des Tabellennamen
    tabname = table + objtyp

    # SQL-Eintragskommando
    insertsql = "\
BEGIN\n\
INSERT INTO %s \
(%s) VALUES(%s);\n\
COMMIT;\nEND;\n/\n"\
% (tabname, join(SQL_Att, ', '), join(SQL_Val, ', '))

    SQL_Kom += insertsql

    return SQL_Kom


PolygonSQL = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2007, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(%s),
MDSYS.SDO_ORDINATE_ARRAY(%s));
"""
def exportPolygon(obj):
    """Erzeugt eine SQL-Kommando zur Definition der Geometrie eines
    Polygonfeatures"""

    # Polygon aus den Linienteilen erzeugen. Bei Mierfolg abbrechen
    polygonliste = obj.get_Polygon()
    if polygonliste == []:
        return ""

    # Index: ORA-spezifisch zur Definition von MDSYS.SDO_ELEM_INFO_ARRAY:
    # Regelt, ab welcher Koordinate Lcher bzw. weitere Polygone definiert
    # werden.
    index = 1
    coords = []
    elements = []

    for polygon in  polygonliste:
        coordlist = polygon.getganzeListe()
        # 1003: Neues Polygon
        elements.append("%s,1003,1" %index)
        index += len(coordlist) * 2

        # nach 30 coord-Paaren ein newline einfgen. Am Anfang ein newline
        # einfgen um evtl. berhang durch vorhergehendes Polygon abzubauen
        coord_anzahl = 30
        # Alle Coordinaten eintragen...
        for (x,y) in coordlist:
            if coord_anzahl == 30:
                endline = '\n'
                coord_anzahl = 0
            else:
                endline = ''
            coords.append('%s,%s%s' %(round(x,3),round(y,3), endline))
            coord_anzahl += 1
        # Die Lcher des Polygons durchgehen
        for loch in polygon.getLoecher():
            lochcoordlist = loch.getganzeListe()
            if len(lochcoordlist) > 2:
                # 2003: Neues Loch
                elements.append("%s,2003,1" %index)
                index += len(lochcoordlist) * 2
                # nach 30 coord-Paaren ein newline einfgen. Am Anfang ein newline
                # einfgen um evtl. berhang durch vorhergehendes Polygon abzubauen
                coord_anzahl = 30
                # Alle Coordinaten eintragen...
                for (x,y) in lochcoordlist:
                    if coord_anzahl == 30:
                        endline = '\n'
                        coord_anzahl = 0
                    else:
                        endline = ''
                    coords.append('%s,%s%s' %(round(x,3),round(y,3), endline))
                    coord_anzahl += 1
            else: 
                print "Ein Loch in diesem Polygon hat weniger als 3 Koordinaten."

    # Eintrag des einzelnen Indizes und Koordinaten in das SQL-Kommando
    SQL_Kom = PolygonSQL % (join(elements,','), join(coords,','))

    return SQL_Kom


LineSQL = """DECLARE
geom mdsys.sdo_geometry :=
MDSYS.SDO_GEOMETRY(2006, null, null, MDSYS.SDO_ELEM_INFO_ARRAY(%s),
MDSYS.SDO_ORDINATE_ARRAY(%s));
"""
def exportLine(obj):
    """Erzeugt ein SQL-Kommando zur Definition der Geometrie eines
    Linienfeatures """

    #Holt sich die Linien
    Linienzugliste = obj.get_ganzeLinie()

    coords = []
    elements = []
    index = 1
    for linie in  Linienzugliste:
        coordlist = linie.getganzeListe()
        elements.append("%s,2,1" %index)
        index += len(coordlist) * 2

        coord_anzahl = 0 
        # Alle Coordinaten eintragen...
        # nach 30 coord-Paaren ein newline einfgen
        for (x,y) in coordlist:
            if coord_anzahl == 30:
                endline = '\n'
                coord_anzahl = 0
            else:
                endline = ''
            coords.append('%s,%s%s' %(round(x,3),round(y,3), endline))
            coord_anzahl += 1

    SQL_Kom = LineSQL % (join(elements,','), join(coords,','))

    return SQL_Kom

def exportPoint(obj):
    """Erzeugt ein SQL-Kommando zur Definition der Geometrie eines
    Punktfeatures"""

    (x, y) = obj.Objkoor
    SQL_Kom = """
    DECLARE
    geom mdsys.sdo_geometry :=
    MDSYS.SDO_GEOMETRY( 2001, null, MDSYS.SDO_POINT_TYPE(%s, %s, null),
    null, null);
    """ % (x, y)

    return SQL_Kom

def exportRahmenobjekt(obj):
    """Gibt den Rahmen als SQL-Geometrie eines Rahmenobjektes
    zurck."""
    coords = obj.get_RahmenobjektPolygon().getganzeListe()
    coordsAsString = map(coordsToString, coords)

    elements = ["1,1003,1"]

    return PolygonSQL %  (','.join(elements), ','.join(coordsAsString))

def coordsToString(xy):
    return '%s,%s' % (round(xy[0],3),round(xy[1],3)) 

def exportObjekt(obj, tableconf):
    """Exportiert ein Objekt ohne die Objektteile als SQL-Kommando. Die
    besonderen Informationen (BI) aus den Objektteilen werden dabei
    bercksichtigt, sofern sie in obj.BIAtt und obj.BIVal bereits geladen
    sind"""

    # Die Objekt-spezifischen Attribute
    SQL_Att = []
    SQL_Val = []
    SQL_Att.append("objnr")
    SQL_Val.append("'%s'" %obj.Objnr)
    SQL_Att.append("objart")
    SQL_Val.append("%s" % obj.art)
    SQL_Att.append("aktualitaet")
    SQL_Val.append("'%s'" %obj.aktualitaet)
    SQL_Att.append("edatum")
    SQL_Val.append("to_date('%s', 'RRMMDD')" % obj.datum)

    # Attribute aus den BI
    SQL_Att += obj.BIAtt
    SQL_Val += obj.BIVal

    # Die Geometrie
    if obj.typ =="F":
        SQL_geom = exportPolygon(obj)
        if SQL_geom == '':
            print "Objekt %s: Die Linien lassen sich nicht zu einem Polygon zusammenfgen" % obj.Objnr
            return ''
    elif obj.typ == "L":
        SQL_geom = exportLine(obj) 
    elif obj.typ == "P":
        SQL_geom = exportPoint(obj)
    elif obj.typ == "R":
        SQL_geom = exportRahmenobjekt(obj)
    else:
        print "Objekt %s: Geometrie-Typ '%s' unbekannt." % (obj.Objnr, obj.typ)
        return ''

    SQL_Kom = SQL_geom

    # Tabellenname erzeugen
    tablekey = (obj.folie,'','')
    if not tableconf.tables.has_key(tablekey):
        print "Tabellenspezifikation fr Folie %s ist nicht vorhanden" % obj.folie
        print obj.folie, obj.art
        return ''
    table = tableconf.tables[tablekey]
    if len(table) > 1:
        print "Es ist nur eine Tabelle fr die Speicherung der Features erlaubt"
    else:
        tabname = '%s_%s' %(table.keys()[0], obj.typ.lower())


    # Erstellen des INSERT_Befehles
    # Der Prozedure Teil des INSERT-Kommandos wird erstellt
    insertsql = "\
BEGIN\n\
    INSERT INTO %s (%s, geom) VALUES(%s, geom);\n\
    COMMIT;\nEND;\n/\n" \
    %(tabname, join(SQL_Att,', '), join(SQL_Val, ', '))

    SQL_Kom += insertsql

    return SQL_Kom

def exportFeature(obj, tableconf):
    """Exportiert ein einzelnes Feature (d.h. Objekt und seine Objektteile)
    als SQL-Kommandos.
    Liefert einen String zurck, der die SQL-Kommandos enthlt"""

    SQL_Kom =""
    if obj.folie != '':
        for ot in obj.objektteile:

            # Spezifikations- Schlssel
            tablekey = (obj.folie, ot.artderInfo, ot.darstellung)
            # Tabellennamen durchgehen
            if tableconf.tables.has_key(tablekey):
                curTableconf = tableconf.tables[tablekey]
                for table in curTableconf.keys():
                    attrSpec = curTableconf[table]['attributes'].split(';')
                    # Speicherung als Objektteil-Tabelle
                    if table != '':
                        SQL_Kom += exportObjektteil(obj, ot, attrSpec, table) 

                    # Erzeugen der Attributliste zum Speichern in der
                    # Objekt-Tabelle
                    else:
                        Attlist, Vallist = \
                            ot.extractAttributes(attrSpec,table)
                        Vallist = map(anpassen, Vallist)
                        obj.BIAtt += Attlist
                        obj.BIVal += Vallist

        SQL_Kom_tmp = exportObjekt(obj, tableconf)
        SQL_Kom += SQL_Kom_tmp
    else:
        log.warning("warning: Objekt mit Nummer: %s hat keine Foliennummer" % obj.Objnr)

    return SQL_Kom

