# -*- encoding: iso-8859-1 -*-
# Copyright (C)  2004 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

import logging
import ogr
from string import join
log = logging.getLogger('')


class FeatureCollection:
    """ Diese Klasse dient dazu, die nach der EDBS-Philosophie
    gehaltenen Daten aus dem ONS in die Tabellenstruktur aus der
    edbsilon-Konfigurationsdatei zu bringen. Anschlieend knnen
    die Daten ber eine mit OGR ansprechbare Connection in ein
    anderes Datenformat exportiert werden
    """ 

    def __init__(self, driver):
        self.tableconf = None
        self.tables = {}
        self.datasource = ''
        self.driver = driver
        self.deletelist = []
        self.ONS = {}

    def createSchema(self):
        """ dummy fuer Featurecollection"""
        pass


    def dropSchema(self):
        """ dummy fuer Featurecollection"""
        pass

    def convertONS(self):
        """Diese Funktion erhlt einen Objektnummernspeicher (ONS)(, der zum
        Beispiel mit second_parsing erzeugt wurde). 
        Zunchste werden alle
        Objekte mit Lsch-bit gelscht, danach werden alle (Neu-)Eintrge in
        die Datenbank abgearbeitet. Die erzeugten SQL-Befehle werden in die
        Datei 'Ziel_SQL' geschrieben.
        """

        # Zuerst werden die Lschoperationen bearbeitet
        #delete(ONS, sqlfile)

        # Fr alle Objnr die bekannt, werden die OGR-Feature erzeugt
        print "Trage %d Objekte ein" % len(self.ONS)
        for Objnr in self.ONS.keys():
            if not self.ONS[Objnr].compareObjnr(Objnr):
                print Objnr , self.ONS[Objnr].compareObjnr(Objnr)  
                raise ValueError("error: Ein objdaten_sammler hat andere Objnr als im ONS steht")
            # Danach werden die INSERT-operationen bearbeitet
            self.readObj(self.ONS[Objnr])


    def readObj(self, obj):
        """Diese Methode erzeugt die  SQL_Kommados fr diesen
        objdaten_sammler.
        Falls obj.Status in ['N','M'], werden alle BesonderenInfos 
        abgearbeitet und das ergebnis zu
        SQL_Kom concated und der SQL_Befehl fr die Folie-Tabelle 
        erstellt und concated"""

        if obj.folie != '':
            for ot in obj.objektteile:

                # Spezifikations- Schlssel
                tablekey = (obj.folie, ot.artderInfo, ot.darstellung)
                # Tabellennamen durchgehen
                if self.tableconf.tables.has_key(tablekey):
                    curTableconf = self.tableconf.tables[tablekey]
                    for table in curTableconf.keys():
                        attrSpec = curTableconf[table]['attributes'].split(';')
                        # Speicherung als Objektteil-Tabelle
                        if table != '':
                            self.exportObjektteil(obj, ot, attrSpec, table) 

                        # Erzeugen der Attributliste zum Speichern in der
                        # Objekt-Tabelle
                        else:
                            Attlist, Vallist = \
                                ot.extractAttributes(attrSpec, table)
                            obj.BIAtt += Attlist
                            obj.BIVal += Vallist
            self.exportObjekt(obj)
        else:
            log.warning("warning: Objekt mit Nummer: %s hat keine Foliennummer" % obj.Objnr)

        if obj.Status in ['L','M']:
            self.deletelist.append(obj.Objnr)


    def delete(self, ds):
        """Diese Funktion geht die ONS durch und sammlet alle Objnr mit
        Lschkommandos. Dann werden die DELETE-Kommados fr die einzelnen
        Tabellen geschrieben"""

        if len(self.deletelist) == 0:
            return
        for i in  range(ds.GetLayerCount()):
            layer =  ds.GetLayer(i)
            layer.GetFeatureCount()
            f = layer.GetNextFeature()
            while f:
                f.GetFieldIndex('objnr')
                objnr = f.GetField(f.GetFieldIndex('objnr'))
                if objnr in self.deletelist:
                    layer.DeleteFeature(f.GetFID())
                f = layer.GetNextFeature()


    def export(self):
        """ Exportieren der Elemente"""

        self.convertONS()
        driver = ogr.GetDriverByName(self.driver)
        ds = driver.Open(self.datasource, 1)
        if ds == None:
            ds = driver.CreateDataSource(self.datasource)
        self.delete(ds)
        for t in self.tables.keys():
            featurelist = self.tables[t]
            if len(featurelist) != 0:
                try:
                    layer = ds.GetLayerByName( t )
                except:
                    # es gibt mindestens ein Objekt in der featurelist.
                    # Setze Geometry-Typ des ersten Objektes als geomtype
                    # fest
                    feature = featurelist[0]
                    if feature.has_key('geom'):
                        geomtype = feature['geom'].GetGeometryType()
                        layer = ds.CreateLayer( t, None, geomtype) 
                    else:
                        layer = ds.CreateLayer( t )
                    for a in self.tables[t][0].keys():
                        if a != 'geom':
                            field = ogr.FieldDefn(a, ogr.OFTString)
                            layer.CreateField(field)
                for f in self.tables[t]:
                    feature = ogr.Feature( layer.GetLayerDefn() ) 
                    for a in f.keys():
                        if a != 'geom':
                            idx = feature.GetFieldIndex(a)
                            feature.SetField(idx, f[a])
                        else:
                            feature.SetGeometry(f[a])
                    layer.CreateFeature(feature)
        ds.Destroy()

    def exportObjekt(self, obj):
        """Exportiert ein Objekt ohne die Objektteile als SQL-Kommando. Die
        besonderen Informationen (BI) aus den Objektteilen werden dabei
        bercksichtigt, sofern sie in obj.BIAtt und obj.BIVal bereits geladen
        sind"""

        # Die Objekt-spezifischen Attribute
        attributes = {}
        
        attributes['objnr'] = obj.Objnr
        attributes['objart'] = obj.art
        attributes['aktualitaet'] = obj.aktualitaet

        monate = []
        monate.append('jan')
        monate.append('feb')
        monate.append('mar')
        monate.append('apr')
        monate.append('may')
        monate.append('jun')
        monate.append('jul')
        monate.append('aug')
        monate.append('sep')
        monate.append('oct')
        monate.append('nov')
        monate.append('dec')
        try:
            tag = obj.tag
            monat = monate[int(obj.monat) -1]
            jahr = int(obj.jahr)
            if jahr < 30:
                attributes['edatum'] = '%s.%s.20%s' %(tag, monat, jahr)
            else:
                attributes['edatum'] = '%s.%s.19%s' %(tag,monat,jahr)
        except:
            pass

        for a in obj.BIAtt:
            attributes[a] = obj.BIVal[obj.BIAtt.index(a)]

        # Geometrie
        Objtyp = obj.typ
        if Objtyp =="F":
            geom = getFlaeche(obj)
        elif Objtyp == "L":
            geom = getLinie(obj) 
        elif Objtyp == "P":
            geom = getPunkt(obj)
        elif Objtyp == "R":
            geom = getRahmenobjekt(obj)
        else:
            geom =''
            print "DEBUG: Unbekannter Objekttyp %s" % Objtyp
        if geom == '':
            print "Objekt %s: Das Objekt hat keine korrekte Geometrie" % obj.Objnr
        else:
            attributes['geom'] = geom

        # Tabellenname erzeugen
        tablekey = (obj.folie,'','')
        if not self.tableconf.tables.has_key(tablekey):
            print "Tabellenspezifikation fr Folie %s ist nicht vorhanden" % obj.folie
            print obj.folie, obj.art
            return ''
        table = self.tableconf.tables[tablekey]
        if len(table) > 1:
            print "Es ist nur eine Tabelle fr die Speicherung der Features erlaubt"
        else:
            tabname = '%s_%s' %(table.keys()[0], obj.typ.lower())
        if not self.tables.has_key(tabname):
            self.tables[tabname] = []
        self.tables[tabname].append(attributes)


    def exportObjektteil(self, obj, ot, attrSpec, table):
        """ Die besonderen Informationen werden als
        Zusatzinformationen zu einem EDBS-Objekt gespeichert. Da
        die besonderen Informationen zustzlich auch
        Geographische Angaben enthalten knnen, wird ein Teil
        u.U. in eigenen Tabellen abgelegt. 
        Dies geschieht in dieser Methode. 
        """

        attributes = {}
        attributes['objnr'] = obj.Objnr
        attributes['kartentyp'] = ot.kartentyp
        attributes['darstellungsart'] = ot.darstellung


        # Attribute aus den besonderen Informationen
        Att_tmp, Val_tmp = \
            ot.extractAttributes(attrSpec,table)

        for a in Att_tmp:
            attributes[a] = Val_tmp[Att_tmp.index(a)]

        # Geodaten, wenn vorhanden, wird auch ein Winkel erzeugt
        objtyp = ''
        # Schaut die Geodaten durch
        if ot.artderGeo in ['51','52','53', '11', '15']:
            geom, winkel = getGeometrie(ot, ot.artderGeo)
            attributes['geom'] = geom
            if winkel != None:
                attributes['winkel'] = winkel

            # Geometry -Attribut
            if ot.artderGeo in ['51','52','53']:
                objtyp = "_p"
            else:
                objtyp = "_l"

        # Erzeugen des Tabellennamen
        tabname = table + objtyp

        if not self.tables.has_key(tabname):
            self.tables[tabname] = []
        self.tables[tabname].append(attributes)
        return {}


def getFlaeche(obj):
    """Gibt die Flche eines Objektes als
    OGR-Polygon-Geometry zurck
    """

    polygonliste = obj.get_Polygon()
    if polygonliste == []:
        return ""
    else:
        multipolygon = []
        for polygon_tmp in polygonliste:
            polygonAsList = []
            coordAsList = []
            for (x,y) in polygon_tmp.getganzeListe():
                coordAsList.append('%s %s' %(round(x,3),round(y,3)))
            polygonAsList.append('(%s)' % join(coordAsList,',') )
            loecher = polygon_tmp.getLoecher()
            if len(loecher) == 0:
                pass
            else:
                for loch in polygon_tmp.getLoecher():
                    coordAsList = []
                    for (x,y) in loch.getganzeListe():
                        coordAsList.append('%s %s' %(round(x,3),round(y,3)))
                    polygonAsList.append('(%s)' % join(coordAsList,',') )
            multipolygon.append('(%s)' % join(polygonAsList, ','))

        return ogr.CreateGeometryFromWkt('MULTIPOLYGON(%s)'
            %join(multipolygon, ','))

def getLinie(obj):
    """Gibt die Linie eines Objektes als
    OGR-LINESTRING-Geometrie zurck
    """
    #Holt sich die Linien
    Linienzugliste = obj.get_ganzeLinie()
    multiline = []
    for Linienzug in Linienzugliste:
        coordAsList = []
        for(x,y) in Linienzug.getganzeListe():
            coordAsList.append('%s %s' %(round(x,3),round(y,3)))
        multiline.append('(%s)' %join(coordAsList,','))
    return ogr.CreateGeometryFromWkt('MULTILINESTRING(%s)' %join(multiline,','))

def getPunkt(obj):
    """Gibt einen Punkt eines Objekes als
    ogr-POINT-Geometrie zurck
    """
    (x, y) = obj.Objkoor
    return ogr.CreateGeometryFromWkt('POINT(%s %s)' %(x,y))

def getRahmenobjekt(obj):
    """Gibt den Rahmen eines Rahmenobjektes als ogr-POLYGON-Geometrie
    zurck."""
    coords = obj.get_RahmenobjektPolygon().getganzeListe()
    coordsAsString = map(coordsToString, coords)
        
    return ogr.CreateGeometryFromWkt('POLYGON((%s))' % ",".join(coordsAsString))

def coordsToString(xy):
    return '%s %s' % (round(xy[0],3),round(xy[1],3))                

def getGeometrie(objektteil, artdergeo):
    """Hier wird aus den Geometrieangaben SQL-Befehle gemacht. Es gibt
       die Mglichkeit fr PostgreSQL(PG) und Oracle DB die passenden
       SQL_Befehle zu schreiben ber die Variable format. Geht
       die artdergeo durch(ob 11,15,51,52,53) und gibt die passenden
       SQL_Befehle zurck.
       """

    # Linie
    if artdergeo in ['11', '15']:

        linie = objektteil.geoangaben()
        linienstr = []
        for (x,y) in linie:
            linienstr.append('%s %s' %(round(x,3),round(y,3)))
        return ogr.CreateGeometryFromWkt('LINESTRING(%s)' %join(linienstr,',')), None

    # Punkt ohne Winkelangabe
    elif artdergeo == '51' or artdergeo == '52':
        (x,y) = objektteil.geoangaben()
        return ogr.CreateGeometryFromWkt('POINT(%s %s)' %(x,y)), None

    # Punkt mit Winkelangabe
    elif artdergeo == '53':
        (winkel,(x, y)) = objektteil.geoangaben()
        return ogr.CreateGeometryFromWkt('POINT(%s %s)' %(x,y)), winkel
