# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2004-2007 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

import sys
import logging
log = logging.getLogger('')

from edbsclasses import Feature, edbs_satz, edbs_satz_liste, edbs_line,\
    II_data

# begin of the preparsing
def first_parsing(file):
    """Sucht nach zusammengehrigen EDBS-Zeilen; sucht die EDBS-Datenstze
    durch und bearbeitet sie je nach Zugehrigkeitsbyte:
    ' ': ist, wird ein Eintrag edbs_satz in plist angehngt
    'A' ist, wird eine edbs_satz Objekt mit edbs_satz dieses Satzes und der
        Folgestze gefllt und an plist angehngt.
    Folgestze sind EDBS-Stze mit Zugehrigkeitsbyte 'F' oder am Ende 'E'.
    """

    plist = []
    line = file.readline()

    if line !="":
        entry_tmp = edbs_satz(line)

    while line != "":
        if entry_tmp.relation() == " ":
            plist += [entry_tmp]
        elif entry_tmp.relation() == "A":
            list_tmp =edbs_satz_liste(entry_tmp)
            line = file.readline()
            if line =="":
                raise ValueError("error: Folge- oder End-EDBS-Satz erwartet") 
            entry_tmp = edbs_satz(line)      
            while entry_tmp.relation() == "F":
                list_tmp.additionalentry(entry_tmp)
                line =file.readline()
                entry_tmp = edbs_satz(line)
            if entry_tmp.relation() == "E":
                list_tmp.additionalentry(entry_tmp)
            else:
                 raise ValueError("error: Andere Zugehoerigkeit erwartet")  
            plist += [ list_tmp]
        else :
            raise ValueError("error: Falsche Zugehoerigkeit")

        line =file.readline()
        entry_tmp = edbs_satz(line)

    return plist

def second_parsing(ONS, eliste, edbsformat):
    """Geht die edbs_satz/edbs_satzliste Liste elist durch und checkt get_entryart
    und verarbeitet den Eintrag der Art entsprechend:
        Art:                  | Was gemacht wir:
        OBJEKTDATEN             Eintrag in den Objektdaten_sammler
        AUFTRAGSKENNDATEN       Eintrag ins Logbuch
        AUFTRAGSTEXTDATEN       Eintrag ins Logbuch
        AENDERUNSDATUM          Eintrag ins Logbuch (nur BZSN)
        OBJLOESCHEN             Eintrag in den Objektdaten_sammler (fr BZSN)
        GEBIETSKENNZEICHNUNG    Pkte werden umgewandelt und ins Logbuch abgelegt
        AUFTRAGSKENNSATZ        Eintrag ins Logbuch
        AUFTRAGSENDE            Eintrag ins Logbuch als "Ende"
        UNDEFINIERT             Eintrag ins Logbuch als "Undefiniert"

    Die Routine gibt den Objektsammelspeicher (ONS) zurck.         
    """

    for e in eliste:
        art = e.get_entryart()

        if art == "OBJEKTDATEN":

            # Parsen der Daten in der EDBS-Zeile
            II_daten = e.get_II_data(edbsformat)

            # Gammadaten: Informationen zum Objekt
            for  gamma in range(len(II_daten.gammalist)): 
                Objnr = II_daten.gammalist[gamma].objfkt[10:17]
                # Objekt anlegen, wenn es noch nicht existiert
                if not ONS.has_key(Objnr):
                    ONS[Objnr]= Feature(Objnr)

                # Objektdaten eintragen
                objDaten = II_daten.gammalist[gamma]
                objPunkt = II_daten.objkoor
                ONS[Objnr].setObjdaten(objDaten, objPunkt)

            # Linienkoordinaten: sie muessen zu den zugehoerigen Objekten
            # gespeichert werden
            for alpha in range( len(II_daten.alphalist) ):
                objKoor = II_daten.objkoor
                endPkt = II_daten.alphalist[alpha].endpoi
                linetype = II_daten.alphalist[alpha].linetype
                lageparam = II_daten.alphalist[alpha].paralist
                linienFkt = II_daten.alphalist[alpha].fktlinelist

                # Linien-Funktion: Zughrigkeit zu rechts- oder
                # linksseitigem Objekt
                for theta in linienFkt:

                    #holte die linke und die rechte Objnr aus II_daten
                    ObjnrL = theta[7:14] 
                    ObjnrR = theta[14:21]
                    folie = theta[0:3]
                    objart = theta[3:7]

                    # Trage Lageparameter in ONS zur Objnr ein, wenn
                    # notwendig lege ein neues Objekt im ONS an.
                    if ObjnrL != '       ': 
                        tmp_line = edbs_line(
                            objKoor, endPkt, linetype, lageparam, 
                            folie, objart)
                        if not ONS.has_key(ObjnrL):
                            ONS[ObjnrL] = Feature(ObjnrL)
                        ONS[ObjnrL].appendline(tmp_line)

                    if ObjnrR != '       ': 
                        lageparam.reverse()
                        tmp_line = edbs_line(
                            endPkt, objKoor,
                            linetype, lageparam,
                            folie, objart)
                        if not ONS.has_key(ObjnrR):
                            ONS[ObjnrR] = Feature(ObjnrR)
                        ONS[ObjnrR].appendline(tmp_line)

        elif art =="AUFTRAGSKENNDATEN":
            II_text = e.get_II()
            log.info("Auftragskenndaten:\n")
            kenndaten = II_text[4:].split(' - ')
            for k in kenndaten:
                log.info("    %s\n" % k.strip())
        elif art == "AUFTRAGSTEXTDATEN":
            II_text = e.get_II()
            log.info("Auftragstextdaten:\n")
            log.info(II_text[4:]) 
        elif art == "AENDERUNSDATUM":
            II_text = e.get_II()
            log.info("nderung am: %s.%s.%s " % (II_text[4:6], 
                                                     II_text[2:4],
                                                     II_text[0:2]))
            log.info("um %s:%s:%s\n" % (II_text[7,9], 
                                             II_text[9,11], 
                                             II_text[11,13]))
        elif art== "OBJDATENLOESCHEN":
            (Objnr, AK) = e.get_SK_Objnr()
            if not ONS.has_key(Objnr):
                ONS[Objnr] = Feature(Objnr)
            ONS[Objnr].loescheObj(AK)

        elif art == "GEBIETSKENNZEICHNUNG":
            II_text = e.get_II()
            log.info("Gebietskennzeichen: \n")
            log.info("    linksunten: (%s )\n" 
                        % II_text[ 0:20])
            log.info("    rechtsoben: (%s )\n" 
                        % II_text[20:40])

        elif art == "AUFTRAGSKENNSATZ":
            II_text = e.get_II()
            log.info("Auftragskennsatz: \n")
            log.info("Dienststelle: \n")
            log.info("    %s\n"% II_text[4:18] ) # Holt die Dienststelle raus
            log.info("    %s\n" %II_text[18: 51])
            log.info("Ausgabeinfo: \n" )
            log.info("    %s\n" %II_text[51:83].strip())
            log.info("Rest des Eintrags: ")
            log.info("    %s\n" %II_text[83:])

        elif art == "AUFTRAGSENDE":
            log.info("Ende\n")

        elif art == "UNDEFINIERT":
            log.info("Undefiniert:\n")
            log.info(str(e))

        else:
            raise ValueError("error: Unerwarteter edbs_satz -Typ")
    return ONS

def parsen(ONS, Quelle, edbsformat):
    """Auslesen eine EDBS-Datei (Quelle).
    Im ersten Schritt werden zusammengehrige EDBS-Satze zusammengefhrt.
    Der zweite Schritt liest die Objekte aus einem EDBS-Auftragssatz und
    legt die Daten in einem Objektdaten-Sammler ab.
    """

    # ffnen der Quellendatei
    quell = open(Quelle ,"r") 
    # Logbucheintrag schreiben: der Header
    log.info("================================================\n")
    log.info("   %s \n" % Quelle)
    log.info("================================================\n")
    # parsen der Daten     
    liste = first_parsing(quell)
    return second_parsing(ONS, liste, edbsformat)

class TableConf:
    """ Klasse zur Speicherung der Tabellenkonfigurationen:
        1. Zuordnung der Besonderen Informationen nach Folie, Objektart und
        Darstellung zu bestimmten Tabellen einschlielich der Informationen
        zum Auslesen 
    """

    def __init__(self, filename):
    
        self.tables = {}
        self.folien = []
        self.tabnamen = []
        self.lese_konfdatei(filename)

    def lese_konfdatei(self, filename):
        """Aus der Datei Konf werden die Daten gezogen um das Directory Spezi
         fr die Spezifikationen aufzubauen.
       Geht folgendermassen vor: Holt sich eine Zeile,
       Splitted die Kommas raus, verknpft die ersten drei zum Schlssel
       und das vierte kommt in Wert  dann wird das sechste
       nochmal bzg ; gesplitted und die diese Attributsliste an Wert appended.
       Dies sind alle Infos fr einen Eintrag in dem Directory Spezi:
       Schlssel und [Tabelle, Attributsliste]. 
       Ausserdem wir eine Liste aller Tabellen angelegt: edbsclasses.Tabnamen"""

        # Einlesen der Konfigurationsdatei
        try:
            konffile = open(filename,"r")
        except:
            print "Konfigurationsdatei konnte nicht geffnet werden."
            print "Dateiname " + filename
            sys.exit()

        konflines = konffile.readlines()
        konffile.close()

        for line in konflines:
            # Ignorieren der Zeile, da Kommentarzeile
            if line[0] =="#" or line[0] == '\n':
                continue

            # Trennung der Zeile nach Schlssel, Tabellenname + -Typ und 
            # Attributspezifikation
            try:
                (keystring, table, attributes) =  line.split("|")
            except:
                print "Tabellenspezifikation nicht korrekt:\n%s" %line

            key1, key2, key3 = keystring.split(',')
            key = (key1, key2, key3)

            # Fge Folie ein
            if key1 not in self.folien:
                self.folien.append(key1)

            if key in self.tables.keys():
                tableconf = self.tables[key]
            else:
                tableconf = {}
                self.tables[key] = tableconf
            tablename = table.split(',')[0]
            tabletypes = table.split(',')[1]


            # Fge Tabellennamen ein
            for typ in tabletypes:
                if tablename == '':
                    break
                if typ == 'n':
                    fulltablename = tablename
                else:
                    fulltablename = "%s_%s" % (tablename, typ)
                if fulltablename not in self.tabnamen:
                    self.tabnamen.append(fulltablename)

            tableconf[tablename] = \
                            {'types':tabletypes, \
                            'attributes': attributes.strip()}
