# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2007 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.


"""test_Polygon.py   von Ludwig Reiter
   begonnen: 28.10.2007
   Tested: Die vier verschiedenen Faelle von ALK Niedersachsen Daten.
   Dafuer wird die get_Polygone Funktion von Feature getestet auf:
   1. Quadrat
   2. Quadrat mit Einschluss
   3. Quadrat mit Hilfslinien-Einschluss
   (auskommentiert) 4. Drei Quadrate mit Hilfslinien"""

# Richtigen Pfade.
import sys
import os
from string import join, split
sys.path +=[os.path.join(os.pardir, "Konverter")]
sys.path +=[os.path.join(os.pardir, "Lib")]

import unittest
import edbsclasses

class Polygons_TestCase(unittest.TestCase):
    """Polygon-Testfaelle."""
    def setUp(self):
        self.a = self.createEdbs_line((0.0,0.0),(0.0,4.0),"1")
        self.b = self.createEdbs_line((0.0,4.0),(4.0,4.0),"1")
        self.c = self.createEdbs_line((4.0,4.0),(4.0,0.0),"1")
        self.d = self.createEdbs_line((4.0,0.0),(0.0,0.0),"1")
        self.e = self.createEdbs_line((0.0,0.0),(2.0,1.0),"1")
        self.f = self.createEdbs_line((1.0,2.0),(0.0,0.0),"1")
        self.g = self.createEdbs_line((2.0,1.0),(1.0,2.0),"1")
        self.h = self.createEdbs_line((4.0,0.0),(2.0,3.0),"0242")
        self.i = self.createEdbs_line((2.0,3.0),(4.0,0.0),"0242")
        self.j = self.createEdbs_line((2.0,3.0),(2.0,2.0),"1")
        self.k = self.createEdbs_line((2.0,2.0),(3.0,3.0),"1")
        self.l = self.createEdbs_line((3.0,3.0),(2.0,3.0),"1")
        self.v1 = self.createEdbs_line((4.0,4.0),(7.0,4.0),"0242")
        self.r2a = self.createEdbs_line((7.0,4.0),(8.0,2.0),"1")
        self.r2b = self.createEdbs_line((8.0,2.0),(6.0,2.0),"1")
        self.r2c = self.createEdbs_line((6.0,2.0),(7.0,4.0),"1")
        self.v2 = self.createEdbs_line((7.0,4.0),(11.0,4.0),"0242")
        self.r3a = self.createEdbs_line((11.0,4.0),(12.0,2.0),"1")
        self.r3b = self.createEdbs_line((12.0,2.0),(10.0,2.0),"1")
        self.r3c = self.createEdbs_line((10.0,2.0),(11.0,4.0),"1")


    def createEdbs_line(self,anfangspkt, endpkt, typ):
        return edbsclasses.edbs_line(anfangspkt, endpkt,"",[],"",typ)

    def tearDown(self):
        pass

    def test_one_quadrat(self):
        """Ein Quadrat mit vermischten Linienzuegen wird versucht mit 
getPolygon zu bearbeiten."""
        feature = edbsclasses.Feature("NR1")
        feature.linelist = [self.a, self.c, self.b, self.d]
        result = feature.get_Polygon()
        self.compareLists(result[0].getganzeListe(), 
                [(0.0,0.0),(0.0,4.0),(4.0,4.0),(4.0,0.0),(0.0,0.0)])

    def compareLists(self, list1, list2): 
        i = 0
        while i < min(len(list1), len(list2)):
            self.assertEqual(list1[i],list2[i])
            i += 1

    def test_quadrat_with_inner_Points(self):
        """Ein Quadrat das einige innere Punkte und an einen Wert mehrfach
        ankommt."""
        feature = edbsclasses.Feature("NR2")
        feature.linelist = [self.d, self.e, self.b, self.g, self.f, self.a, 
                self.c]
        result = feature.get_Polygon()
        self.assertEqual(len(result),1)
        self.compareLists(result[0].getganzeListe(),
                [(4.0,0.0),(0.0,0.0),(2.0,1.0),(1.0,2.0),(0.0,0.0),(0.0,4.0),
                 (4.0,4.0),(4.0,0.0)])

    def test_quadrat_with_hole(self):
        """Ein Quadrat und ein Loch mit Hilfslinien."""
        feature = edbsclasses.Feature("NR3")
        feature.linelist = [self.a, self.b, self.d, 
                self.c, self.h, self.i, self.j, self.k, self.l]
        result = feature.get_Polygon()
        self.assertEqual(len(result),1)
        self.compareLists(result[0].getganzeListe(),
                [(0.0,0.0),(0.0,4.0),(4.0,4.0),(4.0,0.0),(0.0,0.0)])
        self.assertEqual(len(result[0].getLoecher()),1)
        self.compareLists(result[0].getLoecher()[0].getganzeListe(),
                [(2.0,3.0),(2.0,2.0),(3.0,3.0),(2.0,3.0)])

#    def test_quadrat_and_rectangle(self):
#        """Ein Quadrat, eine Hilfslinie, ein Rechteck, noch eine Hilfslinie
#        und schliesslich noch ein Rechteck."""
#        feature = edbsclasses.Feature("NR4")
#        feature.linelist = [self.a,self.b,self.c,self.d,self.v1,self.v2,
#                self.r2a,self.r2b,self.r2c,
#                self.r3a,self.r3b,self.r3c]
#        result = feature.get_Polygon()
#        self.assertEqual(len(result), 3)
#        self.compareLists(result[0].getganzeListe(),
#            [(4.0,4.0),(4.0,0.0),(0.0,0.0),(0.0,4.0),(4.0,4.0)])
#        self.compareLists(result[1].getganzeListe(),
#            [(7.0,4.0),(8.0,2.0),(6.0,2.0),(7.0,4.0)])
#        self.compareLists(result[2].getganzeListe(),
#            [(11.0,4.0),(12.0,2.0),(10.0,2.0),(11.0,4.0)])


