# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2004 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""test_edbsilon.py    von Ludwig Reiter
   begonnen: 19.08.2004
   Testet bisher: * first_parsing Funktion von der Klassen entry
                    und entrylist von konvertor.py
                  * get_II_data Funktion von der Klasse entry
                  * II_data Klasse
                  * second_parsing Funktion
                  * die Feature Klasse                 """

# Zuerst wird versucht die richtigen Pfade zu setzen
import sys
import os
from string import join, split
sys.path +=[os.path.join(os.pardir, "Konverter")]
sys.path +=[os.path.join(os.pardir, "Lib")]

import unittest
import edbsilon
import edbsclasses
import logging
import parse
import exportDB

log = logging.getLogger('')
hdlr = logging.FileHandler('test.log')
log_format = logging.Formatter('%(message)s')
hdlr.setFormatter(log_format)
log.addHandler(hdlr)
log.setLevel(logging.INFO)


class Get_II_TestCase(unittest.TestCase):
    """"Testfall: Hier wird the entry-Methode get_II_data getestet und
         die Methoden die dazugehren. Dabei wird der File ALK_Testdaten.edb
         benutzt und damit richtige ALK-Daten."""
    def setUp(self):
        """Aufbau der Testumgebung: ffnen der Datei ALK_Testdaten.edb"""
        self.file = open("BVB_TEST10001_test.edb","r")

    def tearDown(self):
        """Aufrumen: Schliessen der ALK_Testdaten.edb"""
        self.file.close()

    def test_one_run(self):
        """Dieser Test durchprft die II der Datenstze 2 bis 199 auf
        funtionieren von get_II_data. Zuerst werden die Daten mit Hilfe von
        first_parsing in die Form [E,E,E,..,E] gebracht. E steht fr entry
        und entrylist kommt nicht in ALK_Testdaten vor.
        Dann wird bei den einzelnen entry Objekten geet_II_data() aufgerufen
        und geschaut ob bei der Ausgabe eine betalist vorhanden ist.
        Eine betalist ist die 3. Liste im Ausgabetupel von get_II_data().
        Sie dafr die Methode get_II_data().
        Geprft wird ob betalisten insgesamt vorhaben waren."""
        fplist =parse.first_parsing(self.file)
        anzahl =0
        for x in range(3,40):
            oneentry = fplist[x]
            tmp = oneentry.get_II_data('alk')
            if tmp.gammalist !=[]:
                anzahl +=1
        self.failUnless( anzahl >=1)

    def test_first_nummer(self):
        """Testfall: erste Koordiante von Datensatz 3
        Hat der Datensatz 3 von BVB_TEST10001.edb als
         Grundrisskennnr/Objektkoordinate die richtigen Daten, die ich
         per Hand aus der Datei gelesen habe."""
        fplist = parse.first_parsing(self.file)
        oneentry = fplist[16]
        tmp = oneentry.get_II_data('alk')
        self.assertEqual(tmp.objkoor, (3374657.136 ,5806720.175))

    def test_alphalist(self):
        """Testfall: alpha-list
        Hat der Datensatz 94 von BVB_TEST10001.edb an
        Stelle eins des Ergebnisstupels von get_II_data() die richtigen Daten.
        Ich vergleiche die Ergebnissdaten mit denen die ich per Hand
        rausgelesen habe."""

        fplist = parse.first_parsing(self.file)
        oneentry = fplist[3]
        tmp = oneentry.get_II_data('alk')
        self.assertEqual(tmp.alphalist[0].endpoi, (3374863.001 ,5806888.989))
        self.assertEqual(tmp.alphalist[0].linetype, "15")
        self.assertEqual(tmp.alphalist[0].fktlinelist[0], "0010233       B0282W1  ")
        self.assertEqual(tmp.alphalist[0].paralist[0], (3374847.599 ,5806883.788))

    def test_gammalist(self):
        """Testfall: gamma-list Datensatz 3
        Hat der Datensatz 3 von der Datei BVB_TEST10001.edb
        an Stelle zwei des Ergebnisstupels von get_II_data() die richtigen per
        Hand eingelesenen Daten."""
        fplist =parse.first_parsing(self.file)
        oneentry = fplist[16]
        tmp = oneentry.get_II_data('alk')
        self.assertEqual(tmp.objkoor, (3374657.136 ,5806720.175))
        self.assertEqual(tmp.gammalist[0].objfkt, "052011901PB0282VE030312")
        self.assertEqual(tmp.gammalist[0].betalist[0][0][0], \
                "15NN  0119  33587406202019                   ")


    def test_readWHF(self):
        """Testfall: read_WHF
        Getestet wird, ob read_WHF den pos -Wert (fr Position) richtig erhht
        und richtig einliesst. An der 36-40 Stelle des Datensatzes sollte
        0001 stehen, was 1 ergibt"""
        fplist =parse.first_parsing(self.file)
        oneentry = fplist[3]
        oneentry.pos = 36
        tmp = oneentry.read_WHF()
        self.failUnless( oneentry.pos == 40)
        self.failUnless( tmp == 1)

class SimpleParser_1TestCase(unittest.TestCase):
    """Dieser Testcase testet first_parsing.

       Dabei ist zu beachten, dass er verschieden Pseudo EDBS-Datenstze aufruft.
       testbb: 2 Pseudo-EDBS-Stze mit blank bei der Zugehrigkeit
       testbAEb: 4 Pseudo-EDBS-Stze mit blank,A, E, blank als Zugehrigkeit
       testbAFEb: 5 P.EDBS-Stze mit blank AFE blank als Zugehrigkeit
       testbAFFEb: 6 Stze mit Zugehrigkeit bAFFEb
       ALK_Testdaten.edbs: Richtige Daten aus der ALK-Datei"""

    def setUp(self):
        """Aufbau vor dem Testen:
        ffnet die Testfiles: testbb.edb, testbAEb.edb, testbAFE.edb, testbAFFE.edb
           und BVB_TEST10001.edb"""
        self.filebb = open("testbb.edb","r")
        self.filebAEb = open("testbAEb.edb","r")
        self.filebAFEb = open("testbAFEb.edb","r")
        self.filebAFFEb = open("testbAFFEb.edb","r")
        #self.filereal = open("BVB_TEST10001.edb","r")

    def tearDown(self):
        """Aufrumen: Schliessen der unter Aufbau geffneten Files"""
        self.filebb.close()
        self.filebAEb.close()
        self.filebAFEb.close()
        self.filebAFFEb.close()
        #self.filereal.close()

    def testpassbb(self):
        """Testet first_parsing mit Hilfe von der Datei testbb.edb.
           Lsst sich alle entry und entrylist -Objekte als String geben und
           vergleicht sie mit dem Inhalt des Testfile    """
        self.list= parse.first_parsing(self.filebb)
        self.stringfromlist =""
        for x in self.list:
            self.output = str(x)
            self.stringfromlist += self.output
        self.filebb.seek(0)
        self.tmp = self.filebb.read()
        self.failUnless ( self.tmp == self.stringfromlist )

    def testpassbAEb(self):
        """Siehe testpassbb nur mit Hilfe von file testbAEb.edb"""
        self.list= parse.first_parsing(self.filebAEb)
        self.stringfromlist =""
        for x in self.list:
            self.output = str(x)
            self.stringfromlist += self.output
        self.filebAEb.seek(0)
        self.tmp = self.filebAEb.read()
        self.failUnless ( self.tmp == self.stringfromlist )

    def testpassbAFEb(self):
        """Mit Hilfe von Testfile testbAFEb.edb wird wie in testpassbb getestet"""
        self.list= parse.first_parsing(self.filebAFEb)
        self.stringfromlist =""
        for x in self.list:
            self.output = str(x)
            self.stringfromlist += self.output
        self.filebAFEb.seek(0)
        self.tmp = self.filebAFEb.read()
        self.failUnless ( self.tmp == self.stringfromlist )

    def testpassbAFFEb(self):
        """Mit Hilfe von Testfile testbAFFEb.edb wird wie in testpassbb getestet"""
        self.list= parse.first_parsing(self.filebAFFEb)
        self.stringfromlist =""
        for x in self.list:
            self.output = str(x)
            self.stringfromlist += self.output
        self.filebAFFEb.seek(0)
        self.tmp = self.filebAFFEb.read()
        self.failUnless ( self.tmp == self.stringfromlist )

    #def testpassreal(self):
    #    """Mit Hilfe von Testfile BVB_TEST10001.edb wird wie in testpassbb getestet"""
    #    self.list= parse.first_parsing(self.filereal)
    #   self.stringfromlist =""
    #   for x in self.list:
    #       self.output = str(x)
    #       self.stringfromlist += self.output
    #   self.filereal.seek(0)
    #   self.tmp = self.filereal.read()
    #   self.failUnless ( self.tmp == self.stringfromlist )

class  SimpleParser_2TestCase( unittest.TestCase ):
    """testet die second_parsing Routine
       zuerst wird kurz entry.get_entryart getestet,
       dann ob es entry_art UNDEFINIERT in der elist gibt,
       ob das Ende des Logbuchs wirklich Ende ist,
       dass punkttrans funktioniert"""
    def setUp(self):
        """Aufbau der Testumgebung: ffenen der Datei ALK_Testdaten.edb und
           einmal ausfhren von first_parsing darauf"""
        self.file = open("BVB_TEST10001_test.edb","r")
        self.elist = parse.first_parsing(self.file)

    def tearDown(self):
        """Aufrumen: Schliessen von ALK_Testdaten.edb"""
        self.file.close()

    def testget_entryart(self):
        """vergleicht den Rckgabewert von self.elist[0].get_entryart
           mit  AUFTRAGSKENNSATZ"""
        self.failUnless( self.elist[0].get_entryart() =="AUFTRAGSKENNSATZ")

    def testget_entryart_noUNDEFINIERT(self):
        """Geht die ganze elist durch und schaut, ob UNDEFINIERTe
           entryart auftreten."""
        i =0
        for e in self.elist :
            if e.get_entryart() == "UNDEFINIERT":
                 print "Schau mal bei Stelle : ",i
            self.failUnless( e.get_entryart() != "UNDEFINIERT" )
            i +=1
    def testpunkttrans(self):
        """Testet an Hand des Testwertes 10100101000001000001 ob die Routine
        fuktioniert"""
        self.assertEqual( \
            edbsclasses.punkttrans('10100101000001000001', 'alk'),\
            (1001000.001,1001000.001) )

    def testline(self):
        """Testet die Klasse line, indem ein line-Objekt mal angelegt wird
        und der Konstruktor getestet"""
        linie1 = edbsclasses.edbs_line(0,1,2,[0,1,2],4,5)
        linie2 = edbsclasses.edbs_line(1,0,2,[2,1,0],4,5)
        self.assertEqual(linie1.Anfangspkt, 0)
        self.assertEqual(linie2.Anfangspkt, 1)
        self.assertEqual(linie2.Endpkt, 0)
        self.assertEqual(linie2.paralist, [2,1,0] )

    def test_O_second_parsing(self):
        """hier wird getestet, ob die Routine second_parsing richig mit den
         OBJEKTDATEN umgeht, indem die BVB_TEST10001.edb ber second_parsing
         bearbeitet werden und gschaut wird ob der Schlssel 'B01T673'
         im ONS vorhanden ist und ein Feature hat das den selben Schlssel
         hat."""
        self.ONS = parse.second_parsing({}, self.elist, 'alk')
        self.failUnless(self.ONS.has_key('B01T673') )
        self.failUnless(self.ONS['B01T673'].compareObjnr('B01T673') )

    def test_L_second_parsing(self):
        """hier wird getestet, ob der ONS fr das erste Objekt im Daten die
          richtigen Daten hat: Es wird ONS['G002R7T'].linelist[0] und [1]
          verglichen mit den richtigen Daten"""
        self.ONS = parse.second_parsing({}, self.elist,'alk')

        self.failUnless(self.ONS.has_key('B01T7PL'))
        obj = self.ONS['B01T7PL']
        self.failUnless(obj.compareObjnr('B01T7PL') )
        #self.assertEqual(len(obj.linelist), 4)
        #self.assertEqual(obj.linelist[0].get_Anfangspkt(), \
            #"33567795946003037100")
        #self.assertEqual(obj.linelist[1].get_Anfangspkt(), \
            #"33567795530690125901")

    def test_second_parsing_bug(self):
        """Bei einer Zeilekomplex nach AFE in den Testdaten klappte das
           zweite -Parsing nicht. Diesen Fall baue ich hier nach um ihn gut zu
           testen."""

        e1 = edbsclasses.edbs_satz("EDBS19560000FEIN016481A 0000ULO8ALK 00010001335877066696009946003000000010001011133101FB01T2030303090049000116NN  1031HA12069304009260052     01       530002TT11263             33587706669600994600000117NN  1031HA12069304009260064     01       530002TT11257             33587707575100013700000117NN  1031HA12069304009260062     01       530002TT11257             33587707590400009300000117NN  1031HA12069304009260060     01       530002TT11269             33587707606500007400000117NN  1031HA12069304009260058     01       530002TT11269             33587707622300002900000117NN  1031HA12069304009260056     01       530002TT11263             33587707638500000900000117NN  1031HA12069304009260054     01       530002TT11263             33587706653900996500000131K4  1045                                 1100023358770758098800253433587707584239000553000131K4  1045                                 1100023358770662852899295633587706632047993267000131K4  1045                                 1100023358770662483399370133587706627899991757000131K4  1045                                 1100023358770662457699242733587706628155993031000131K4  1045                                 1100023358770661273999613733587706615806994194000131K4  1045                                 1100023358770661248399486333587706616063995468000131K4  1045                                 1100023358770660924399786133587706612367996212000131K4  1045                                 1100023358770660904599688233587706612565997192000131K4  1045                                 1100023358770759716000029633587706600284998647000131K4  1045                                 1100023358770659696299931633587706600482999627000131K4  1045                                 1100023358770759326800006033587706596333998117000131K4  1045                                 1100023358770659301199878633587706596590999391000131K4  1045                                 11000233587707565396005676335877075694050058880001")
        e2 = edbsclasses.edbs_satz("EDBS19770000FEIN016481F 0000ULO8ALK 31K4  1045                                 1100023358770756559300665633587707569209004908000131K4  1045                                 1100023358770757699000334133587707580999003552000131K4  1045                                 1100023358770757718700432033587707580802002572000131K4  1045                                 1100023358770758091700122433587707584496001827000131K4  1014                                 1100023358770664062199051933587706643943989850000131K4  1014                                 1100023358770664405998850233587706647381987832000131K4  1014                                 1100023358770665615298606433587706659474985395000131K4  1014                                 1100023358770666010498659533587706663916985826000131K4  1014                                 1100023358770667169798425833587706675509983491000131K4  1045                                 1100023358770667189598523933587706675509983491000131K4  1045                                 1100023358770667169798425833587706675707984470000131K4  1045                                 1100023358770666030198757433587706663916985826000131K4  1045                                 1100023358770666010498659533587706664113986807000131K4  1045                                 1100023358770665640898733933587706659474985395000131K4  1045                                 1100023358770665615298606433587706659731986669000131K4  1045                                 1100023358770664431698977633587706647381987832000131K4  1045                                 1100023358770664405998850233587706647638989106000131K4  1045                                 1100023358770664081899150033587706643943989850000131K4  1045                                 1100023358770664062199051933587706644141990831000131K4  1045                                 1100023358770662872599393733587706631850992287000131K4  1014                                 11000233587707565396005676335877075692090049080001")
        e3 = edbsclasses.edbs_satz("EDBS07640000FEIN016481E 0000ULO8ALK 31K4  1014                                 1100023358770757699000334133587707580802002572000131K4  1014                                 1100023358770758091700122433587707584239000553000131K4  1014                                 1100023358770659301199878633587706596333998117000131K4  1014                                 1100023358770659696299931633587706600284998647000131K4  1014                                 1100023358770660904599688233587706612367996212000131K4  1014                                 1100023358770661248399486333587706615806994194000131K4  1014                                 1100023358770662457699242733587706627899991757000131K4  1014                                 1100023358770662852899295633587706631850992287")
        eliste = edbsclasses.edbs_satz_liste(e1)
        eliste.additionalentry(e2)
        eliste.additionalentry(e3)
        II_data = eliste.get_II_data('alk')

    def testget_SK_Objnr(self):
        """Legt ein entry-Objekt an und schaut wie die Objektnr und die
           Aktualitt aus den SK geholt wird holt werden.
           Getestet wird ob die Objnr = 'OBJNR07' und die AK= 'AT' ist.
           Der Rest des EDBS-Satzes ist unwichtig."""
        e =edbsclasses.edbs_satz("AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGGGGHASDLOB2005EQOBJNR07ANDLOB2003EQAT")
        e2=edbsclasses.edbs_satz("AAAAABBBBBCCCCCDDDDDEEEEEFFFFFGGGGGHASDLOB2003EQATANDLOB2005EQOBJNR07")
        self.failUnless( e.get_SK_Objnr() == ('OBJNR07' , 'AT') )
        self.failUnless(e2.get_SK_Objnr() == ('OBJNR07' , 'AT') )

   # def testget_ganzeLinie(self):
   #     """Teste ob das Objekt G002PH2 passende viele Linienelemente (12)hat
   #     und ob das dritte Linienelement passenden Anfangswert."""
   #     (Logbuch_tmp, ONS_tmp) = edbsilon.second_parsing(self.elist)
   #     sammler = ONS_tmp["G002PH3"]
   #     self.failUnless( len(sammler.linelist) ==7)

class II_data_TestCase( unittest.TestCase ):
    """testet die II_data Klasse auf Funktionalitt.

       getestet werden der Konstruktor und die get Methoden:
       get_Objektkoor(self),get_EndpktderLinie(self, alpha),
       get_FktderLinieliste(self, alpha), get_Lageparaliste(self, alpha) ,
       get_Lageparaliste(self, alpha),get_FktdesObj,
       get_BesondereInfo(obj, gamma, beta),
       get_Geometrieangabenliste(self, gamma, beta)"""

    def setUp(self):
        """Aufbau der Testumgebung: ffnen der Datei BVB_TEST10001.edb"""
        self.file = open("BVB_TEST10001_test.edb","r")
        self.elist = parse.first_parsing(self.file)
        self.II_data1 = self.elist[3].get_II_data('alk')
        self.II_data2 = self.elist[16].get_II_data('alk')

    def tearDown(self):
        """Aufrumen: Schliessen der BVB_TEST10001.edb"""
        self.file.close()

    def testget_Objektkoor(self):
        """Testet: ob die Objektkoordiaten des Objektes self.II_data1
           richtig zurckgegeben wird"""
        self.assertEqual(self.II_data1.objkoor,\
            (3374812.746, 5806872.019))

    def testget_EndpktderLinie(self):
        """Vergleicht bei self.II_data1 get_EndpktderLinie mit per Hand
            Daten und bei self.II_data2 get_EndpktderLinie ''   """

        self.assertEqual(self.II_data1.alphalist[0].endpoi, \
            (3374863.001, 5806888.989))
        self.assertEqual(self.II_data2.alphalist, [])

    def testget_FktderLinieliste(self):
        """Vergleicht self.II_data1 bei get_FktderLinie
           zurckgegebenens mit Hand eingelesenen"""
        self.assertEqual(self.II_data1.alphalist[0].fktlinelist, ["0010233       B0282W1  "])

    def testget_Lageparaliste(self):
        """Vergleicht, ob [] ausgeben wurde bei self.II_data2 und
           vergleicht, ob self.II_data1.get_Lageparaliste mit per Hand
           Daten bereinstimmt"""
        self.assertEqual(self.II_data1.alphalist[0].paralist, \
            [(3374847.599, 5806883.788) ])
        self.assertEqual(self.II_data2.alphalist, [])


class ring_TestCase(unittest.TestCase):

    def setUp(self):
        # Ein Polygon
        self.ring1 = edbsclasses.Ring([(0,0),(0,1),(1,1),(1,0),(0,0)])
        # Ein Loch
        self.ring2 = edbsclasses.Ring([(0,0),(1,0),(1,1),(0,1),(0,0)])

    def test_isPolygon(self):
        self.failUnless(self.ring1.isPolygon())
        self.failUnless(not self.ring2.isPolygon())

class Feature(unittest.TestCase):
    """Testet die Funktionen von Feature , die zu wandel gehren.
       get_ganzeLinie(), get_Polynom(),get_geoangaben """


    def setUp(self):
        """Aufbau: Legt einige line und Feature -Objekte an"""
        # (0 0),(1 0)
        line1 = edbsclasses.edbs_line((0.0, 0.0), (1.0, 0.0), "",[],"","")
        # (1 0),(2 0),(2 1)
        line2 = edbsclasses.edbs_line((1.0, 0.0), (2.0, 1.0),"", [(2.0, 0.0)],"","")
        # (2 1),(1 1),(0 1)
        line3 = edbsclasses.edbs_line((2.0, 1.0),(0.0, 1.0),"", [(1.0, 1.0)],"","")
        # (0 1), (0 0)
        line4 = edbsclasses.edbs_line((0.0, 1.0), (0.0, 0.0), "",[],"","")
        # (1.0 0),(2 0)
        line5 = edbsclasses.edbs_line((1.0, 0.0), (2.0, 0.0), "",[],"","")
        # (2 0),(1 0)
        line6 = edbsclasses.edbs_line((2.0, 0.0),(1.0, 0.0), "",[],"","")
        # (1 0), (0 0)
        line7 = edbsclasses.edbs_line((1.0, 0.0),(0.0, 0.0), "",[],"","")
        # (0 0), (0 3)
        line8 = edbsclasses.edbs_line((0.0, 0.0),(0.0, 3.0), "",[],"","")
        # (0 3.0),(1 2) tempLinie
        line9 = edbsclasses.edbs_line((0.0, 3.0),(1.0, 2.0), "",[],"","0242")
        # (1 2),(1 1)
        line10 = edbsclasses.edbs_line((1.0, 2.0),(1.0, 1.0), "",[],"","")
        # (1 1), (2 1)
        line11 = edbsclasses.edbs_line((1.0, 1.0),(2.0, 1.0), "",[],"","")
        # (2 1),(2 2)
        line12 = edbsclasses.edbs_line((2.0, 1.0),(2.0, 2.0), "",[],"","")
        # (2 2),(1 2)
        line13 = edbsclasses.edbs_line((2.0, 2.0,),(1.0, 2.0), "",[],"","")
        # (1 2),(0 3) tempLinie
        line14 = edbsclasses.edbs_line((1.0, 2.0),(0.0, 3.0), "",[],"","0242")
        # (0 3.0), (3 3), (3 0), (0 0)
        line15 = edbsclasses.edbs_line((0.0, 3.0),(0.0, 0.0), "",\
            [(3.0, 3.0),(3.0, 0)],"","")
        # (0 3), (3 3), (0 3)
        line16 = edbsclasses.edbs_line((0.0, 3.0),(2.0, 0.0), "",\
            [(3.0, 3.0),(3.0, 0.0)],"","")

        self.col1 = edbsclasses.Feature('TESTnr1')
        self.col2 = edbsclasses.Feature('TESTnr2')
        self.col3 = edbsclasses.Feature('TESTnr3')
        self.col4 = edbsclasses.Feature('TESTnr4')
        self.col5 = edbsclasses.Feature('TESTnr5')

        # col1 ist ne echte Linie
        self.col1.appendline(line1)
        self.col1.appendline(line2)
        self.col1.appendline(line3)

        # col2 ist ein echtes Polygon
        self.col2.appendline(line6)
        self.col2.appendline(line7)
        self.col2.appendline(line8)
        self.col2.appendline(line16)

        # col3 ist ein echtes Polygon mit unsortierter Reihenfolge der
        # Linien
        self.col3.appendline(line6)
        self.col3.appendline(line8)
        self.col3.appendline(line7)
        self.col3.appendline(line16)

        #col4 fr den Anfangspkt passt und Endpkt passt bug
        self.col4.appendline(line1)
        self.col4.appendline(line7)
        self.col4.appendline(line5)
        self.col4.appendline(line6)

        #col5 mit Loch
        self.col5.appendline(line8)
        self.col5.appendline(line9)
        self.col5.appendline(line10)
        self.col5.appendline(line11)
        self.col5.appendline(line12)
        self.col5.appendline(line13)
        self.col5.appendline(line14)
        self.col5.appendline(line15)

    def testget_ganzeLinie(self):
        """Schaut, ob die Ergebnissliste von self.col1.get_ganzeLinie()
        die richtigen Pkte in der richtigen Reihenfolge hat."""
        Linienzugliste = self.col1.get_ganzeLinie()
        self.failUnless( Linienzugliste[0].getganzeListe()  == [(0.0,0.0),(1.0,0.0),(2.0,0.0),
                (2.0,1.0),(1.0,1.0),(0.0,1.0)])

    def testget_Polygon(self):
        """Schaut, ob die Ergebnissliste von self.col2.get_Polygon()
        die richtige Pktreihenfolge besitzt"""
        pktliste_tmp = self.col2.get_Polygon()[0].getganzeListe()

        self.assertEqual( pktliste_tmp, [(2.0, 0.0), (1.0, 0.0), (0.0, 0.0), (0.0, 3.0), (3.0, 3.0), (3.0, 0.0), (2.0, 0.0)])

    def testget_Polygon_bug(self):
        """Schaut, ob die Ergebnissliste von self.col3.get_Polygon()
        den gleichen Anfangs- und Endpunkt besitzt"""
        pktliste_tmp = self.col3.get_Polygon()[0].getganzeListe()
        self.assertEqual( pktliste_tmp, [(2.0, 0.0), (1.0, 0.0), (0.0, 0.0), (0.0, 3.0), (3.0, 3.0), (3.0, 0.0), (2.0, 0.0)])

        self.failUnless( pktliste_tmp[0] == pktliste_tmp[-1])


    def testget_Polygon_Loch(self):
        """Schaut, ob die Funktion get_Polygon() mit einen Loch im Polygon umgehen
         kann."""
        liste = self.col5.get_Polygon()
        self.failUnless(len (liste[0].getganzeListe() ) == 5)
        self.failUnless(len (liste[0].getLoecher()[0].getganzeListe() ) ==5)

    def testget_Polygon_ZweiPolygone(self):
        """schaut,ob get_Polygon() mit zwei Polygonen klarkommt"""

        # (0 0),(1 1)
        line1 = edbsclasses.edbs_line((0.0, 0.0),(1.0, 1.0),
                       "",[(0.0, 1.0)],"","")
        # (1 1), (2 1) tmpLinie
        line2 = edbsclasses.edbs_line((1.0, 1.0),(2.0, 1.0),
                       "",[],"","0242")
        # (2 1),(2 1)
        line3 = edbsclasses.edbs_line((2.0, 1.0),(2.0, 1.0),
                       "",[(3.0, 1.0),(3.0, 0.0),(2.0, 0.0)],"","")
        # (2 1),(1 1) temp Linie
        line4 = edbsclasses.edbs_line((2.0, 1.0),(1.0, 1.0),
                       "",[],"","0242")
        # (1 1),(0 0)
        line5 = edbsclasses.edbs_line((1.0, 1.0),(0.0, 0.0),
                       "",[(1.0, 0.0)],"","")
        col = edbsclasses.Feature('TESTnr6')
        col.appendline(line1)
        col.appendline(line2)
        col.appendline(line3)
        col.appendline(line4)
        col.appendline(line5)

        liste =col.get_Polygon()

        self.assertEqual(len(liste), 2)
        poly1 = liste[0].getganzeListe()
        poly2 = liste[1].getganzeListe()
        self.failUnless(len (poly1) == 5)
        self.failUnless(len (poly2) == 5)
        self.assertEqual(poly1, [(2,1),(3,1),(3,0),(2,0),(2,1)])
        self.assertEqual(poly2, [(0,0),(0,1),(1,1),(1,0),(0,0)])

    def testget_Polygon_Aussenwinkel(self):
        """Testet, ob bei get_Polygon() bei den Fllen mit den Aussenwinkel
        richtig durchgeht. Dieser Fall besteht, wenn man mit den Innenwinkeln nicht
        mehr weiterkommt."""
        # (1 1), (0 1), (0 0)
        line1 = edbsclasses.edbs_line((0.0, 0.0),(1.0, 1.0),
                       "",[(0.0, 1.0)],"","")
        # (1 1), (2 1), (2 2), (1 2), (1 1)
        line2 = edbsclasses.edbs_line((1.0, 1.0),(1.0, 1.0),
                       "",[(1.0, 2.0),
                       (2.0, 2.0),(2.0, 1.0) ],"","")

        # (0 0),(1 0), (1 1) 
        line3 = edbsclasses.edbs_line((1.0, 1.0),(0.0, 0.0),
                       "",[(1.0, 0.0)],"","")
        col = edbsclasses.Feature('TESTnr7')
        col.appendline(line1)
        col.appendline(line2)
        col.appendline(line3)
        liste = col.get_Polygon()
        
        self.failUnless ( len( liste[0].getganzeListe() ) == 9 )

    def testget_Polygon_Winkel(self):
        """es wird getestet, ob die Winkelfunktion richtig mit
        get_Polygon arbeitet. Schaut sich ein
        zusammengepresstes Klammerpolygon an."""
        # (0 0),(3 0), (3 3)
        line1 = edbsclasses.edbs_line((3.0, 3.0),(0.0, 0.0),
                       "",[(3.0, 0.0),],"","")
        # (3 3), (2 1), (1 2), (3 3) 
        line2 = edbsclasses.edbs_line((3.0, 3.0),(3.0, 3.0),
                       "",[(1.0, 2.0),(2.0, 1.0)],"","")
        # (3 3), (0 3), (0 0)
        line3 = edbsclasses.edbs_line((0.0, 0.0),(3.0, 3.0),
                       "",[(0.0, 3.0)],"","")
        col = edbsclasses.Feature('TESTnr7')
        col.appendline(line1)
        col.appendline(line2)
        col.appendline(line3)
        liste =col.get_Polygon()

        self.assertEqual( liste[0].getganzeListe(),
        [(3,3),(3,0),(0,0),(0,3),(3,3),(1,2),(2,1),(3,3)])

    def test_winkel(self):
        """Testet die winkel -Funktion von edbs_line"""
        # (0 0),(1 1)
        line1 = edbsclasses.edbs_line((0.0, 0.0),(1.0, 1.0),
                       "",[],"","")
        # (0 0),(1 2)
        line2 = edbsclasses.edbs_line((0.0, 0.0),(1.0, 2.0),
                       "",[],"","")
        # (2 1),(1 1)
        line3 = edbsclasses.edbs_line((2.0, 1.0),(1.0, 1.0),
                       "",[],"","")

        self.failUnless( line1.winkel( (0,2), line2 ,"I"))
        self.failUnless( not line2.winkel( (0,2), line1, "I"))
        self.failUnless( line1.winkel( (2,3), line2 ,"I"))
        self.failUnless( not line2.winkel( (2,3), line1, "I"))
        self.failUnless( line1.winkel( (1,0), line2 ,"I"))
        self.failUnless( not line2.winkel( (2,0), line1, "I"))
        self.failUnless( line3.winkel( (1,0), line1, "I"))

    def testget_geoangaben(self):
        """Legt fnf Test Feature -Objekte an und schaut, ob
        jeweils das richige zurckgegeben wird.Die nr nach den col sagt fr
        welche Art der Geometrieangaben es ist."""

        #Erzeugen von Feature-Obj
        col11 = edbsclasses.Feature("0000011")
        col15 = edbsclasses.Feature("0000015")
        col51 = edbsclasses.Feature("0000051")
        col52 = edbsclasses.Feature("0000052")
        col53 = edbsclasses.Feature("0000053")

        II_data11 = edbsclasses.Gamma()
        II_data11.objfkt = "Fk"
        II_data11.betalist = [(['000000000000000000000000000000000000000000011'],\
            ['000000000000000000000', '00000000001000000000',\
            '00000000000500000000'])]

        II_data15 = edbsclasses.Gamma()
        II_data15.objfkt = "Fk"
        II_data15.betalist = [(['000000000000000000000000000000000000000000015'],\
            ['000000000000000000000', '00000000000000000000',\
            '00000000000500000000'])]
        
        II_data51 = edbsclasses.Gamma()
        II_data51.objfkt = "Fk"
        II_data51.betalist = [(['000000000000000000000000000000000000000000051'],\
             ['000000000000000000000'])]
        II_data53 = edbsclasses.Gamma()
        II_data53.objfkt = "FK"
        II_data53.betalist = [(['000000000000000000000000000000000000000000053'],\
            ['TT38000             ', '00000000000000000000'])]

        #Zuordnen der II_data den Feature -Obj
        col11.setObjdaten(II_data11,(0,0))
        col15.setObjdaten(II_data15,(0,0))
        col51.setObjdaten(II_data51,(0,0))
        col52.setObjdaten(II_data51,(0,0))
        col53.setObjdaten(II_data53,(0,0))
        geo11 = col11.objektteile[0].geoangaben()
        geo15 = col15.objektteile[0].geoangaben()
        geo51 = col51.objektteile[0].geoangaben()
        geo52 = col52.objektteile[0].geoangaben()
        geo53 = col53.objektteile[0].geoangaben()
        self.assertEqual(geo11, [(0.0,0.0),(0.5,0.0),(1.0,0.0)])
        self.assertEqual(geo15, [(0.0,0.0),(0.5,0.0),(0.0,0.0)])
        self.assertEqual(geo51, (0.0,0.0))
        self.assertEqual(geo52, (0.0,0.0))
        self.assertEqual(geo53, [38.0 *9, (0.0,0.0)])


class Third_runTestCase( unittest.TestCase):
    """Hier werden die Fehler verifiziert die bei Objdaten_sammler und
    wandel auftreten."""

    def testObjnrG002PH2(self):
        """Verifiziert den Fehler mit der falschen Art der Geo"""

        e =edbsclasses.edbs_satz("EDBS01650000FEIN000187  0000ULO8ALK 00010001335677958155193997523000000010001021611002FG002PH20112200001000131K4  6110                                 51000133567795815519399752")
        ONS = {}
        parse.second_parsing(ONS, [e], 'alk')
        obj =  ONS["G002PH2"]
        self.failUnless(obj.compareObjnr("G002PH2"))
        #self.assertEqual(obj.II_daten[1][0][0][0][0:2], "31")
        self.assertEqual(obj.objektteile[0].artderInfo, "31")
