# -*- encoding: iso-8859-1 -*-
#
# Copyright (C)  2004 by Intevation GmbH
# Authors:
# Ludwig Reiter <ludwig@intevation.de>
# Silke Reimer <silke@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""
Testet das korrekte Auslesen der Konfigurationsdatei
"""

# Zuerst wird versucht die richtigen Pfade zu setzen
import sys
import os
sys.path +=[os.path.join(os.pardir, "Konverter")]
sys.path +=[os.path.join(os.pardir, "Lib")]

import unittest
import edbsilon
from parse import TableConf

class KonfdateiTestCase( unittest.TestCase):
    """ Testet das Parsen der Konfigurationsdatei """

    def test_Konfdatei(self):
        """Testet das Einlesen der Tabellenkonfiguration"""

        tableconf = TableConf("land.konf")

        tables = tableconf.tables

        # Die Schlssel fr Folientabellen 
        # (Element 2 und 3 der Schlssel sind leer) 
        # und BI werden korrekt gelesen
        self.assert_(('001','','') in tables.keys(), \
                "('001','','') not in %s" %tables.keys())
        self.assert_(('001','13','0233') in tables.keys(), \
                "('001','13','0233') not in %s" %tables.keys())
        self.assert_(('002','16','0232') in tables.keys(), \
                "('001','16','0232') not in %s" %tables.keys())

        # Die richtien Tabellennamen werden eingetragen.
        self.assertEquals(tables[('001','','')].keys(), ['f001'])

        # Alle Tabellennamen einer Folien-Objektart-Darstellungs-
        # Kombination erfasst
        for table in tables[('001','13','0233')].keys():
            self.assert_(table in  ['','f001_kennung'], \
                "%s not in ['','f001_kennung']" %table)
        self.assertEqual(len(tables[('001','13','0233')].keys()), 2)

        # berprfung der Inhalte der Tabellenkonfigurationen
        # fr die Folie
        f001AttrSpec = {'types':'f','attributes':''}
        self.assertEquals(f001AttrSpec, tables['001','','']['f001'])

        # berprfung der Inhalte der Tabellenkonfigurationen
        # fr Objektteile
        f001AttrSpec = {'types':'nlp',\
            'attributes':'bundesland,3:4;gemarkung,5:8;flurnr,9:11;zaehler,12:16;nenner,17:19'}
        self.assertEquals(f001AttrSpec, tables['001','13','0239']['f001_kennung'])
        f001AttrSpec = {'types':'f',\
            'attributes':'fkz,3:19'}
        self.assertEquals(f001AttrSpec, tables['001','13','0239'][''])

        self.assertEquals(['001', '002'], tableconf.folien)

        self.assertEquals(['f001_f', 'f001_kennung', 'f001_kennung_l', \
            'f001_kennung_p', 'f001_haken_p', 'f001_pfeil_l',\
            'f001_fachattribute', 'pfeil_l', 'f002_p', \
            'f002_l', 'f002_f', 'f002_kennung', 'f002_kennung_p', \
            'f002_flurnr_p', 'f002_flurnr_l', 'f002_gemarkung_p'], \
            tableconf.tabnamen)


